@@ -2,15 +2,15 @@
 /*
  * xa_acodec.c
  *
- * Copyright (C) 1994,1995,1996,1997 by Mark Podlipec.
+ * Copyright (C) 1994-1997,1998 by Mark Podlipec.
  * All rights reserved.
  *
- * This software may be freely copied, modified and redistributed without
- * fee for non-commerical purposes provided that this copyright notice is
- * preserved intact on all copies and modified copies.
+ * This software may be freely used, copied and redistributed without
+ * fee for non-commerical purposes provided that this copyright
+ * notice is preserved intact on all copies.
  *
  * There is no warranty or other guarantee of fitness of this software.
- * It is provided solely "as is". The author(s) disclaim(s) all
+ * It is provided solely "as is". The author disclaims all
  * responsibility and liability with respect to this software's usage
  * or its effect upon hardware or computer systems.
  *
@@ -50,6 +50,9 @@ xaULONG XA_ADecode_DVIM_PCMxM();
 xaULONG XA_ADecode_DVIS_PCMxM();
 xaULONG XA_ADecode_IMA4M_PCMxM();
 xaULONG XA_ADecode_IMA4S_PCMxM();
+#ifdef GSM
+xaULONG XA_ADecode_GSMM_PCMxM();
+#endif
 
 xaULONG XA_IPC_Sound();
 xaULONG XA_Add_Sound();
@@ -1484,6 +1487,85 @@ xaULONG ocnt,buff_size;
   return(ocnt);
 }
 
+#ifdef GSM
+
+#include "xa_gsm_state.h"
+
+static xaSHORT gsm_buf[320];
+static XA_GSM_STATE gsm_state;
+
+void GSM_Init()
+{
+  memset((char *)(&gsm_state), 0, sizeof(XA_GSM_STATE));
+  gsm_state.nrp = 40;
+}
+
+/***************************************************************************
+ *
+ *
+ */ 
+xaULONG XA_ADecode_GSMM_PCMxM(snd_hdr,obuf,ocnt,buff_size)
+XA_SND *snd_hdr;
+xaUBYTE *obuf;
+xaULONG ocnt,buff_size;
+{ XA_ADECODE_DECLARE_LOCAL; 
+  xaULONG out_cnt = 320;
+  if (snd_hdr==0) return(ocnt);
+  XA_ADECODE_INIT_LOCAL;	/* Init Local Variables */
+  /* Optionally Init Codec Specific Structure Here */ 
+  while(ocnt < buff_size)
+  {
+    if (inc_cnt < (1<<24))
+    {
+      if (snd_hdr->flag == 0)
+      { xaULONG ret, icnt;
+
+/* Quicktime and AVI used different formats */
+	if (spec & 0x80)
+           XA_MSGSM_Decoder(&gsm_state, ibuf, gsm_buf, &icnt, &out_cnt);
+	else
+           XA_GSM_Decoder(&gsm_state, ibuf, gsm_buf, &icnt, &out_cnt);
+	ibuf += icnt;
+	byte_cnt += icnt;
+      }
+
+	/*** Decode Sample and increment byte_cnt ***/
+      if (snd_hdr->flag < out_cnt)  dataL  = (xaLONG)gsm_buf[ snd_hdr->flag ];
+      else /* ERROR */ dataL = 0;
+
+      snd_hdr->flag++; if (snd_hdr->flag >= out_cnt) snd_hdr->flag = 0;
+      samp_cnt--;
+      inc_cnt += inc;
+    }
+
+    while(inc_cnt >= (1<<24))
+    { 
+        /*** Output Sample ***/
+      if (spec & 4)  /* 1 byte output */
+      { xaUBYTE d0 = ((xaULONG)(dataL) >> 8) & 0xff;
+        if (spec & 8)    *obuf++ = xa_sign_2_ulaw[ (xaULONG)(d0) ];
+        else             *obuf++ = (spec & 2)?(d0 ^ 0x80):(d0);
+      }
+      else
+      { xaUBYTE d1,d0; d1 = (dataL>>8) & 0xff; d0 = dataL & 0xff;
+        if (spec & 0x02) d1 ^= 0x80;
+        if (spec & 0x01) {*obuf++ = d0; *obuf++ = d1; }
+        else {*obuf++ = d1; *obuf++ = d0; }
+      }
+      ocnt++;
+      inc_cnt -= (1<<24);	
+      if (ocnt >= buff_size) break;
+    }
+    if (samp_cnt <= 0)		{ XA_ADECODE_MOVE_ON; }
+  }
+  XA_ADECODE_SAVE_LOCAL;     /* save local variables */
+  /* Optionally Save Codec Specific Structure Here */
+  return(ocnt);
+}
+#endif
+
+
+
 /*+_+_+_+_+_+_+_+___+_+_+_+_+_+_+_+_+_++_+_++_+_+_++_+_+++_+_+_+_+_+*/
 /*+_+_+_+_+_+_+_+___+_+_+_+_+_+_+_+_+_++_+_++_+_+_++_+_+++_+_+_+_+_+*/
 /*+_+_+_+_+_+_+_+___+_+_+_+_+_+_+_+_+_++_+_++_+_+_++_+_+++_+_+_+_+_+*/
@@ -1650,6 +1732,26 @@ xaULONG sampsblock;
     }
     totsamps = isamps = tmp_len;
   }
+#ifdef GSM
+  else if (   ((itype & XA_AUDIO_TYPE_MASK) == XA_AUDIO_GSM)
+           || ((itype & XA_AUDIO_TYPE_MASK) == XA_AUDIO_MSGSM)
+	  )
+  { xaLONG w,tmp_len;
+    new_snd->blk_size = blockalign;
+
+     /** calculate total samples in entire chunk for timing purposes */
+    w = (ilen / blockalign);
+    tmp_len = sampsblock * w;           /* out len based on full blocks */
+
+    totsamps = isamps = tmp_len;
+
+    /* POD CHECK for partials??? */
+    if ( (w * blockalign) != ilen)
+    { fprintf(stderr,"GSM weird err. ilen %d block %d w %d quant %d\n",
+                        ilen, blockalign, w, (w * blockalign) );
+    }
+  }
+#endif
   else
   {
     if (itype & XA_AUDIO_STEREO_MSK) bps *= 2;
@@ -1811,6 +1913,14 @@ fprintf(stderr,"XA_IPC_Sound itype %x\n",itype);
 	        new_snd->spec = 2 | 4 | 8;
 		new_snd->delta = XA_ADecode_IMA4S_PCMxM;
 		break;
+#ifdef GSM
+          case XA_AUDIO_GSM_M:
+          case XA_AUDIO_MSGSM_M:
+                new_snd->spec = 2 | 4 | 8;
+		if (itype == XA_AUDIO_MSGSM_M) new_snd->spec |= 0x80;
+                new_snd->delta = XA_ADecode_GSMM_PCMxM;
+                break;
+#endif
 	  case XA_AUDIO_NOP:
 	        new_snd->spec = 2 | 4 | 8;
 		new_snd->delta = XA_ADecode_NOP_PCMXM;
@@ -1910,6 +2020,16 @@ fprintf(stderr,"XA_IPC_Sound itype %x\n",itype);
 		else	new_snd->spec = 1;
 		new_snd->delta = XA_ADecode_IMA4S_PCMxM;
 		break;
+#ifdef GSM
+	  case XA_AUDIO_GSM:
+	  case XA_AUDIO_MSGSM:
+		if (xa_audio_hard_type & XA_AUDIO_BIGEND_MSK)
+			new_snd->spec = 0;
+		else	new_snd->spec = 1;
+		if (itype == XA_AUDIO_MSGSM_M) new_snd->spec |= 0x80;
+		new_snd->delta = XA_ADecode_GSMM_PCMxM;
+		break;
+#endif
 	  case XA_AUDIO_NOP:
 		if (xa_audio_hard_type & XA_AUDIO_BIGEND_MSK)
 			new_snd->spec = 0;
@@ -2023,6 +2143,14 @@ fprintf(stderr,"XA_IPC_Sound itype %x\n",itype);
 		new_snd->spec = 2 | 4;  /* 1 byte output */
 		new_snd->delta = XA_ADecode_IMA4S_PCMxM;
 		break;
+#ifdef GSM
+	  case XA_AUDIO_GSM:
+	  case XA_AUDIO_MSGSM:
+		new_snd->spec = 2 | 4;  /* 1 byte output */
+		if (itype == XA_AUDIO_MSGSM_M) new_snd->spec |= 0x80;
+		new_snd->delta = XA_ADecode_GSMM_PCMxM;
+		break;
+#endif
 	  case XA_AUDIO_NOP:
 		new_snd->spec = 2 | 4;  /* 1 byte output */
 		new_snd->delta = XA_ADecode_NOP_PCMXM;
@@ -2119,6 +2247,16 @@ fprintf(stderr,"XA_IPC_Sound itype %x\n",itype);
 		else	new_snd->spec = 3;
 		new_snd->delta = XA_ADecode_IMA4S_PCMxM;
 		break;
+#ifdef GSM
+	  case XA_AUDIO_GSM:
+	  case XA_AUDIO_MSGSM:
+		if (xa_audio_hard_type & XA_AUDIO_BIGEND_MSK)
+			new_snd->spec = 2;
+		else	new_snd->spec = 3;
+		if (itype == XA_AUDIO_MSGSM_M) new_snd->spec |= 0x80;
+		new_snd->delta = XA_ADecode_GSMM_PCMxM;
+		break;
+#endif
 	  case XA_AUDIO_NOP:
 		if (xa_audio_hard_type & XA_AUDIO_BIGEND_MSK)
 			new_snd->spec = 2;
