@@ -2,18 +2,21 @@
 /*
  * xa_act.c
  *
- * Copyright (C) 1990,1991,1992,1993,1994,1995,1996,1997 by Mark Podlipec. 
+ * Copyright (C) 1990-1997,1998 by Mark Podlipec. 
  * All rights reserved.
  *
- * This software may be freely copied, modified and redistributed without
- * fee for non-commerical purposes provided that this copyright notice is
- * preserved intact on all copies and modified copies.
+ * This software may be freely used, copied and redistributed without
+ * fee for non-commerical purposes provided that this copyright 
+ * notice is preserved intact on all copies.
  * 
  * There is no warranty or other guarantee of fitness of this software.
- * It is provided solely "as is". The author(s) disclaim(s) all
+ * It is provided solely "as is". The author disclaims all
  * responsibility and liability with respect to this software's usage
  * or its effect upon hardware or computer systems.
  *
+ * 17Mar98:  Fixed bug in Make_Images with some types of Clips and TrueColor
+ *	     clip_mask would get mapped to common color. Need to generate
+ *	     clip masks prior to mapping.
  */
 #include "xanim.h"
 #include <Intrinsic.h>
@@ -211,28 +214,54 @@ xaULONG already_disp_flag;	/* ipic already in display form */
     case XA_DIRECTCOLOR:
     case XA_TRUECOLOR:
         inpix_size = x11_bytes_pixel;
-	if (clip_flag == xaTRUE) 
-	  clip_mask = (chdr)?( chdr->map[clip_mask - chdr->coff] ): 0 ;
+
+
+	/* Create Bitmap based on Original GIF Image */
+  if ( (clip_ptr==0) && (clip_flag==xaTRUE) )
+  {
+    clip_ptr = (xaUBYTE *) malloc(ysize * (X11_Get_Bitmap_Width(xsize)/8) );
+    if (clip_ptr==0) TheEnd1("Setup_Mapped: clip malloc err\n");
+    UTIL_Create_Clip(clip_ptr,ipic,clip_mask,xsize,ysize,
+			1,X11_Get_Bitmap_Width(xsize),X11_LSB);
+  }
+
 	if (need_to_scale == xaTRUE)
         {
 	  xaULONG xpos_o,ypos_o,xsize_o,ysize_o;
+
 	  xpos_o = xpos; ypos_o = ypos;
 	  if (full_ipic_flag == xaFALSE) xpos = ypos = 0;
 	  t_pic = UTIL_Alloc_Scaled(xsize,ysize,width,height,
 			buff_x,buff_y,x11_bytes_pixel,xpos_o,ypos_o,xaFALSE);
 	  if (t_pic)
-	  {
+	  { 
+
+	    if ((clip_ptr) && (clip_flag==xaTRUE))
+	    { xaULONG cxpos, cypos, cxsize, cysize;
+	      xaUBYTE *new_clip = UTIL_Scale_Bitmap(0,clip_ptr,
+			xsize,ysize,(X11_Get_Bitmap_Width(xsize/8)),
+			width, height,
+			buff_x, buff_y,
+			&cxpos, &cypos, &cxsize, &cysize,
+			X11_LSB, X11_LSB);
+	      if (new_clip) { free(clip_ptr); clip_ptr = new_clip; }
+	      else { free(clip_ptr); clip_ptr = 0; clip_flag = xaFALSE; }
+	    }
+
 	    if (already_disp_flag==xaTRUE) chdr=0;
 	    UTIL_Scale_Mapped(t_pic,ipic,xpos,ypos,xsize,ysize,
 		inline_size,width,height,buff_x,buff_y,x11_bytes_pixel,
 		&xpos_o,&ypos_o,&xsize_o,&ysize_o,chdr);
             xpos = xpos_o; ypos = ypos_o; xsize = xsize_o; ysize = ysize_o;
 	    act->type = ACT_DISP;	act_map_hdr->psize = 8;
+
 	  }
 	  else /* scale to zero size */
 	  {
-	    act->type = ACT_NOP;	clip_flag = xaFALSE;
+	    act->type = ACT_NOP;	
 	    FREE(act_map_hdr,0x101);	act_map_hdr = 0;
+	    clip_flag = xaFALSE;
+	    if (clip_ptr) { free(clip_ptr); clip_ptr = 0; }
 	  }
 	}
 	else
@@ -263,6 +292,16 @@ xaULONG already_disp_flag;	/* ipic already in display form */
     case XA_STATICCOLOR:
     case XA_GRAYSCALE:
     case XA_PSEUDOCOLOR:
+
+	/* Create Bitmap based on Original GIF Image */
+  if ( (clip_ptr==0) && (clip_flag==xaTRUE) )
+  {
+    clip_ptr = (xaUBYTE *) malloc(ysize * (X11_Get_Bitmap_Width(xsize)/8) );
+    if (clip_ptr==0) TheEnd1("Setup_Mapped: clip malloc err\n");
+    UTIL_Create_Clip(clip_ptr,ipic,clip_mask,xsize,ysize,
+			1,X11_Get_Bitmap_Width(xsize),X11_LSB);
+  }
+
 	if (need_to_scale == xaTRUE)
         {
 	  xaULONG xpos_o,ypos_o,xsize_o,ysize_o;
@@ -273,6 +312,18 @@ xaULONG already_disp_flag;	/* ipic already in display form */
 		buff_x,buff_y,x11_bytes_pixel,xpos_o,ypos_o,xaFALSE);
 	  if (t_pic)
 	  {
+	    if ((clip_ptr) && (clip_flag==xaTRUE))
+	    { xaULONG cxpos, cypos, cxsize, cysize;
+	      xaUBYTE *new_clip = UTIL_Scale_Bitmap(0,clip_ptr,
+			xsize,ysize,(X11_Get_Bitmap_Width(xsize/8)),
+			width, height,
+			buff_x, buff_y,
+			&cxpos, &cypos, &cxsize, &cysize,
+			X11_LSB, X11_LSB);
+	      if (new_clip) { free(clip_ptr); clip_ptr = new_clip; }
+	      else { free(clip_ptr); clip_ptr = 0; clip_flag = xaFALSE; }
+	    }
+
 	    UTIL_Scale_Mapped(t_pic,ipic,xpos,ypos,xsize,ysize,
 			inline_size,width,height,buff_x,buff_y,
 			inpix_size,&xpos_o,&ypos_o,&xsize_o,&ysize_o,0);
