@@ -2,15 +2,15 @@
 /*
  * xa_cmap.c
  *
- * Copyright (C) 1992,1993,1994,1995,1996,1997 by Mark Podlipec. 
+ * Copyright (C) 1992-1997,1998 by Mark Podlipec. 
  * All rights reserved.
  *
- * This software may be freely copied, modified and redistributed without
- * fee for non-commerical purposes provided that this copyright notice is
- * preserved intact on all copies and modified copies.
+ * This software may be freely used, copied and redistributed without
+ * fee for non-commerical purposes provided that this copyright
+ * notice is preserved intact on all copies.
  * 
  * There is no warranty or other guarantee of fitness of this software.
- * It is provided solely "as is". The author(s) disclaim(s) all
+ * It is provided solely "as is". The author disclaims all
  * responsibility and liability with respect to this software's usage
  * or its effect upon hardware or computer systems.
  *
@@ -980,9 +980,9 @@ xaULONG *csize;
 
     if (x11_display_type == XA_MONOCHROME) { size = 256; disp_bits = 8; }
     else
-    {
+    { int xsize = (x11_cmap_size > 256)?(256):(x11_cmap_size);
       size = 0x01; disp_bits = 0;
-      while(size <= x11_cmap_size) { size <<= 1; disp_bits++; }
+      while(size <= xsize) { size <<= 1; disp_bits++; }
       size >>=1; disp_bits--;
     }
     r_bits = 3; g_bits = 3; b_bits = 3; last = 2;
@@ -1062,6 +1062,7 @@ xaULONG *csize;
 {
   xaULONG i,size;
 
+
   if ( (cmap_gray_chdr == 0) || (cmap_cur_gamma != cmap_gray_gamma) )
   {
     xaULONG disp_bits;
@@ -1069,9 +1070,9 @@ xaULONG *csize;
     /* find number of bits in display or use 256 if monochrome */
     if (x11_display_type == XA_MONOCHROME) { disp_bits = 8; size = 256; }
     else
-    {
+    { int xsize = (x11_cmap_size > 256)?(256):(x11_cmap_size);
       size = 0x01; disp_bits = 0;
-      while(size <= x11_cmap_size) { size <<= 1; disp_bits++; }
+      while(size <= xsize) { size <<= 1; disp_bits++; }
       size >>=1; disp_bits--;
     }
     xa_gray_bits = disp_bits;
