@@ -2,15 +2,15 @@
 /*
  * xa_color.c
  *
- * Copyright (C) 1995,1995,1996,1997 by Mark Podlipec.
+ * Copyright (C) 1994-1997,1998 by Mark Podlipec.
  * All rights reserved.
  *
- * This software may be freely copied, modified and redistributed without
- * fee for non-commerical purposes provided that this copyright notice is
- * preserved intact on all copies and modified copies.
+ * This software may be freely used, copied and redistributed without
+ * fee for non-commerical purposes provided that this copyright
+ * notice is preserved intact on all copies.
  *
  * There is no warranty or other guarantee of fitness of this software.
- * It is provided solely "as is". The author(s) disclaim(s) all
+ * It is provided solely "as is". The author disclaims all
  * responsibility and liability with respect to this software's usage
  * or its effect upon hardware or computer systems.
  *
@@ -1465,7 +1465,8 @@ XA_YUV_DECLR;
 	DEC_YUV_DITH(*yp0++,V2R,U2B,UV2G,map_flag,map,chdr,*ip0++);
 	re >>= 1; ge >>= 1; be >>= 1;
     } /* end mx */
-    image += imagex << 1; ybuf += i_x; ubuf += (i_x >> 1); vbuf += (i_x >> 1);
+    image += imagex << 1; ybuf += i_x << 1;
+    ubuf += (i_x >> 1); vbuf += (i_x >> 1);
   } /* end y */
 } /* end function */
 
@@ -1619,7 +1620,7 @@ XA_YUV_DECLR;
 
     ip1 = ip0; yp1 = yp0; up1 = up0; vp1 = vp0;
     if (my > 1)
-      { ip1 += imagex;  yp1 += i_x; up1 += i_x >> 2; vp1 += i_x >> 2; my -= 2; }
+      { ip1 += imagex;  yp1 += i_x; up1 += i_x >> 1; vp1 += i_x >> 1; my -= 2; }
     else my = 0;
     while(x--)
     {   xaULONG iU,iV;
@@ -1641,7 +1642,7 @@ XA_YUV_DECLR;
 	re >>= 1; ge >>= 1; be >>= 1;
     } /* end mx */
     image += (imagex << 1); ybuf +=  (i_x << 1); 
-    ubuf += (i_x >> 1); vbuf += (i_x >> 1);
+    ubuf += i_x; vbuf += i_x;  /* two rows */
   } /* end y */
 } /* end function */
 
