@@ -2,15 +2,15 @@
 /*
  * xa_iff.c
  *
- * Copyright (C) 1990,1991,1992,1993,1994,1995,1996,1997 by Mark Podlipec. 
+ * Copyright (C) 1990-1997,1998 by Mark Podlipec. 
  * All rights reserved.
  *
- * This software may be freely copied, modified and redistributed without
- * fee for non-commerical purposes provided that this copyright notice is
- * preserved intact on all copies and modified copies.
+ * This software may be freely used, copied and redistributed without
+ * fee for non-commerical purposes provided that this copyright
+ * notice is preserved intact on all copies.
  * 
  * There is no warranty or other guarantee of fitness of this software.
- * It is provided solely "as is". The author(s) disclaim(s) all
+ * It is provided solely "as is". The author disclaims all
  * responsibility and liability with respect to this software's usage
  * or its effect upon hardware or computer systems.
  *
@@ -570,7 +570,7 @@ XA_ANIM_HDR *anim_hdr;
 	      if (dlta_hdr == 0) IFF_TheEnd1("IFF_Read_DLTA: malloc err");
 	      act->data = (xaUBYTE *)dlta_hdr;
 	      dlta_hdr->flags = ACT_DBL_BUF;
-	      dlta_hdr->fpos  = xin->Seek_FPos(xin);
+	      dlta_hdr->fpos  = xin->Get_FPos(xin);
 	      dlta_hdr->fsize = chunk.size;
 	      xin->Seek_FPos(xin,chunk.size,1);
 	      if (chunk.size > iff_max_fvid_size) iff_max_fvid_size =chunk.size;
