@@ -2,15 +2,15 @@
 /*
  * xa_ipc.c
  *
- * Copyright (C) 1995,1996,1997 by Mark Podlipec. 
+ * Copyright (C) 1995-1997,1998 by Mark Podlipec. 
  * All rights reserved.
  *
- * This software may be freely copied, modified and redistributed without
- * fee for non-commerical purposes provided that this copyright notice is
- * preserved intact on all copies and modified copies.
+ * This software may be freely used, copied and redistributed without
+ * fee for non-commerical purposes provided that this copyright
+ * notice is preserved intact on all copies.
  * 
  * There is no warranty or other guarantee of fitness of this software.
- * It is provided solely "as is". The author(s) disclaim(s) all
+ * It is provided solely "as is". The author disclaims all
  * responsibility and liability with respect to this software's usage
  * or its effect upon hardware or computer systems.
  *
@@ -690,7 +690,7 @@ XtInputId  *id;
       case XA_IPC_AUD_PORT:
 	audiof->port = ipc_cmd.value;
 	AUD_DEBUG fprintf(stderr,"AUD IPC: AUD_PORT\n");
-	XA_Set_Output_Port(ipc_cmd.value);
+	if (XA_Set_Output_Port) XA_Set_Output_Port(ipc_cmd.value);
         XA_Audio_Send_ACK(XA_IPC_ACK_OK,ipc_cmd.id,0);
 	break;
 
