@@ -2,15 +2,15 @@
 /*
  * xa_jpg.c
  *
- * Copyright (C) 1994,1995,1996,1997 by Mark Podlipec.
+ * Copyright (C) 1994-1997,1998 by Mark Podlipec.
  * All rights reserved.
  *
- * This software may be freely copied, modified and redistributed without
- * fee for non-commerical purposes provided that this copyright notice is
- * preserved intact on all copies and modified copies.
+ * This software may be freely used, copied and redistributed without
+ * fee for non-commerical purposes provided that this copyright
+ * notice is preserved intact on all copies.
  *
  * There is no warranty or other guarantee of fitness of this software.
- * It is provided solely "as is". The author(s) disclaim(s) all
+ * It is provided solely "as is". The author disclaims all
  * responsibility and liability with respect to this software's usage
  * or its effect upon hardware or computer systems.
  *
@@ -462,7 +462,8 @@ xaULONG jpg_skip_marker()
 { xaLONG len,tmp;
   JJ_INPUT_xaSHORT(len); 
   DEBUG_LEVEL1 fprintf(stderr,"SKIP: marker %x len %x\n",jpg_marker,len);
-  len -= 2; if (JJ_INPUT_CHECK(len)==xaFALSE) return(xaFALSE);
+  len -= 2;  if (len <=0) return(xaFALSE);
+  if (JJ_INPUT_CHECK(len)==xaFALSE) return(xaFALSE);
   while(len--) JJ_INPUT_xaBYTE(tmp); /* POD improve this */
   return(xaTRUE);
 }
@@ -1456,6 +1457,7 @@ XA_DEC2_INFO *dec_info;
   xaLONG *qtab0,*qtab1,*qtab2;
   xaUBYTE *Ybuf,*Ubuf,*Vbuf;
   xaULONG rst_count;
+  xaULONG rst_skip = 0;
   xaULONG orow_size = imagex * dec_info->bytes_pixel;
   void (*color_func)() = (void (*)())XA_MCU211111_Func(dec_info->image_type);
 
@@ -1483,13 +1485,48 @@ XA_DEC2_INFO *dec_info;
     while(x--)
     { /* DEBUG_LEVEL1 fprintf(stderr,"MCU XY(%d,%d)\n", x,mcu_rows); */
 
+if (rst_skip)
+{
+  DEBUG_LEVEL1 fprintf(stderr,"  RST_SKIP: rst_skip\n");
+  rst_skip--;
+  memset(Ybuf,0,(DCTSIZE2 << 1));
+  memset(Ubuf,0x80,DCTSIZE2);
+  memset(Vbuf,0x80,DCTSIZE2);
+  Ybuf += (DCTSIZE2 << 1);
+  Ubuf += DCTSIZE2;
+  Vbuf += DCTSIZE2;
+}
+else
+{
+
       JPG_HANDLE_RST(jpg_rst_interval,rst_count);
 
       jpg_huffparse(&jpg_comps[0],jpg_dct_buf,qtab0,Ybuf); Ybuf += DCTSIZE2;
       jpg_huffparse(&jpg_comps[0],jpg_dct_buf,qtab0,Ybuf); Ybuf += DCTSIZE2;
       jpg_huffparse(&jpg_comps[1],jpg_dct_buf,qtab1,Ubuf); Ubuf += DCTSIZE2;
       jpg_huffparse(&jpg_comps[2],jpg_dct_buf,qtab2,Vbuf); Vbuf += DCTSIZE2;
-      JPG_TST_MARKER(jpg_rst_interval,rst_count);
+
+/*      JPG_TST_MARKER(jpg_rst_interval,rst_count); */
+      if (jpg_marker)
+      { DEBUG_LEVEL4 fprintf(stderr,"  jpg_marker(%x)\n",jpg_marker);
+        if (jpg_marker == M_EOI) { jpg_saw_EOI = xaTRUE; }
+        else if ( !((jpg_marker >= M_RST0) && (jpg_marker <= M_RST7)))
+        { fprintf(stderr,"JPEG: unexp marker(%x)\n",jpg_marker);
+          return(xaFALSE);
+        }
+        else
+        { DEBUG_LEVEL1
+                fprintf(stderr,"JPEG: marker %x  rst_cnt %x rst_int %x\n",
+                        jpg_marker,rst_count,jpg_rst_interval);
+          jpg_comps[0].dc = jpg_comps[1].dc = jpg_comps[2].dc = 0;
+          rst_skip = rst_count;
+          rst_count = jpg_rst_interval;
+          jpg_marker = 0;
+          jpg_h_bbuf = 0;  jpg_h_bnum = 0;
+        }
+      }
+
+}
     } /* end of mcu_cols */
 
     (void)(color_func)(iptr,imagex,xaMIN(imagey,8),(mcu_cols * DCTSIZE2),
