@@ -2,15 +2,15 @@
 /*
  * xa_movi.c
  *
- * Copyright (C) 1996,1997 by Mark Podlipec. 
+ * Copyright (C) 1996,1997,1998 by Mark Podlipec. 
  * All rights reserved.
  *
- * This software may be freely copied, modified and redistributed without
- * fee for non-commerical purposes provided that this copyright notice is
- * preserved intact on all copies and modified copies.
+ * This software may be freely used, copied and redistributed without
+ * fee for non-commerical purposes provided that this copyright
+ * notice is preserved intact on all copies.
  * 
  * There is no warranty or other guarantee of fitness of this software.
- * It is provided solely "as is". The author(s) disclaim(s) all
+ * It is provided solely "as is". The author disclaims all
  * responsibility and liability with respect to this software's usage
  * or its effect upon hardware or computer systems.
  *
@@ -20,6 +20,8 @@
  * Revision
  *
  *
+ * 01Dec97  Added support for uncompressed RGB Video Codec.
+ * 18Mar98  Added support for uncompressed MVC2 Video Codec.
  ********************************/
 
 
@@ -52,6 +54,8 @@ extern void JPG_Setup_Samp_Limit_Table();
 
 /* CODEC ROUTINES */
 xaULONG MOVI_Decode_MVC1();
+xaULONG MOVI_Decode_MVC2();
+xaULONG MOVI_Decode_RGB32();
 extern xaULONG JFIF_Decode_JPEG();
 extern xaULONG FLI_Decode_BLACK();
 extern void XA_Gen_YUV_Tabs();
@@ -85,10 +89,11 @@ static xaULONG movi_audio_chans;
 static xaULONG movi_audio_bps;
 xaULONG XA_Add_Sound();
 
-#define MOVI_ICOMP_UNK	0
-#define MOVI_ICOMP_MVC1	1
-#define MOVI_ICOMP_MVC2	2
-#define MOVI_ICOMP_JPEG	10
+#define MOVI_ICOMP_UNK		0
+#define MOVI_ICOMP_MVC1		1
+#define MOVI_ICOMP_MVC2		2
+#define MOVI_ICOMP_JPEG		10
+#define MOVI_ICOMP_RGB32	55
 
 #define MOVI_ACOMP_UNK	0
 #define MOVI_ACOMP_100	100
@@ -322,6 +327,7 @@ xaULONG size;
   { char tbuf[256]; char *p = tbuf;
     while(size--) *p++ = (char)(xin->Read_U8(xin) & 0xff);
     if (strcmp(tbuf,"1") == 0)		ret = MOVI_ICOMP_MVC1;
+    else if (strcmp(tbuf,"2") == 0)	ret = MOVI_ICOMP_RGB32;
     else if (strcmp(tbuf,"10") == 0)	ret = MOVI_ICOMP_JPEG;
     else if (strcmp(tbuf,"MVC2") == 0)	ret = MOVI_ICOMP_MVC2;
     else
@@ -626,6 +632,7 @@ MOVI_HDR *movi_hdr;
       fprintf(stdout, "  Video: %dx%d  codec ", i_hdr->width, i_hdr->height);
       switch(i_hdr->compression)
       { case MOVI_ICOMP_MVC1: fprintf(stdout,"MVC1"); break;
+	case MOVI_ICOMP_RGB32: fprintf(stdout,"RGB"); break;
 	case MOVI_ICOMP_MVC2: fprintf(stdout,"MVC2"); break;
 	case MOVI_ICOMP_JPEG: fprintf(stdout,"JPEG"); break;
 	default: fprintf(stdout,"unk(%d)",i_hdr->compression);
@@ -664,13 +671,22 @@ MOVI_HDR *movi_hdr;
 	JPG_Setup_Samp_Limit_Table(anim_hdr);
         break;
 
-      case MOVI_ICOMP_MVC2:
-        movi->compression = MOVI_ICOMP_UNK;
-        movi->depth = 8;
+      case MOVI_ICOMP_RGB32:
+        movi->compression = MOVI_ICOMP_RGB32;
+        movi->depth = 24;
         movi->imagex = i_hdr->width;
         movi->imagey = i_hdr->height;
-        fprintf(stderr,"  MOVI: Video Codec MVC2 not supported\n"); 
-        return(xaFALSE);
+	/* same */
+        break;
+
+      case MOVI_ICOMP_MVC2:
+        movi->compression = MOVI_ICOMP_MVC2;
+        movi->depth = 16;
+        movi->imagex = i_hdr->width;
+        movi->imagey = i_hdr->height;
+        movi->imagex = 4 * ((movi->imagex + 3)/4);
+        movi->imagey = 4 * ((movi->imagey + 3)/4);
+	JPG_Setup_Samp_Limit_Table(anim_hdr);
         break;
 
       default:
@@ -796,6 +812,15 @@ MOVI_HDR *movi_hdr;
         movi->imagey = i_hdr->height;
         movi->imagex = 4 * ((movi->imagex + 3)/4);
         movi->imagey = 4 * ((movi->imagey + 3)/4);
+	JPG_Setup_Samp_Limit_Table(anim_hdr);
+        break;
+
+      case 2:
+        movi->compression = MOVI_ICOMP_RGB32;
+        movi->depth = 24;
+        movi->imagex = i_hdr->width;
+        movi->imagey = i_hdr->height;
+	/* same */
         break;
 
       default:
@@ -908,17 +933,17 @@ XA_ANIM_HDR *anim_hdr;
 XA_ANIM_SETUP *movi;
 MOVI_HDR *movi_hdr;
 { MOVI_I_HDR *i_hdr = movi_hdr->i_hdr;
+  XA_ACTION *act = 0;
   int i;
 
   if (i_hdr == 0) return;
   for(i = 0; i < i_hdr->frames; i++)
   { xaULONG dlta_len;
-    XA_ACTION *act;
     ACT_DLTA_HDR *dlta_hdr;
   
     dlta_len = i_hdr->size[i];
 
-    act = ACT_Get_Action(anim_hdr,ACT_DELTA);
+    if (act == 0) act = ACT_Get_Action(anim_hdr,ACT_DELTA);
     if (xin->load_flag & XA_IN_LOAD_FILE)
     { dlta_hdr = (ACT_DLTA_HDR *) malloc(sizeof(ACT_DLTA_HDR));
       if (dlta_hdr == 0) TheEnd1("MOVI: dlta malloc err");
@@ -938,7 +963,22 @@ MOVI_HDR *movi_hdr;
       dlta_hdr->fpos = 0; dlta_hdr->fsize = dlta_len;
       ret = xin->Seek_FPos(xin, i_hdr->off[i], 0);
       ret = xin->Read_Block(xin,dlta_hdr->data,dlta_len);
-      if (ret < dlta_len) { fprintf(stderr,"MOVI: read err\n"); return; }
+      if (ret < dlta_len)
+      {
+        if (xa_verbose) fprintf(stderr,
+		"MOVI: read err of frame %d: pos %d size %d (%d)\n",
+		i,i_hdr->off[i],dlta_len,ret);
+	if ((movi->compression == MOVI_ICOMP_RGB32) && (ret > 0))
+	{ 
+	  dlta_hdr->fsize = ret;
+	}
+	else 
+	{ 
+	  free(dlta_hdr);
+	  act->data = 0;
+	  continue;
+	}
+      }
     }
     MOVI_Add_Frame(movi->vid_time,movi->vid_timelo,act);
     dlta_hdr->xpos = dlta_hdr->ypos = 0;
@@ -958,6 +998,18 @@ MOVI_HDR *movi_hdr;
 	dlta_hdr->xapi_rev = 0x0001;
 	dlta_hdr->delta = MOVI_Decode_MVC1;
 	break;
+      case MOVI_ICOMP_MVC2:
+	dlta_hdr->special = 0;
+	dlta_hdr->extra = (void *)(0x0);
+	dlta_hdr->xapi_rev = 0x0001;
+	dlta_hdr->delta = MOVI_Decode_MVC2;
+	break;
+      case MOVI_ICOMP_RGB32:
+	dlta_hdr->special = 0;
+	dlta_hdr->extra = (void *)(0x0);
+	dlta_hdr->xapi_rev = 0x0001;
+	dlta_hdr->delta = MOVI_Decode_RGB32;
+	break;
       default:
 	dlta_hdr->special = 0;
 	dlta_hdr->extra = (void *)(0x0);
@@ -966,7 +1018,9 @@ MOVI_HDR *movi_hdr;
 	break;
     }
     ACT_Setup_Delta(movi,act,dlta_hdr,xin);
+    act = 0; /* indicate we've used action pointer */
   } /* end of i thru frames */
+  if (act) ACT_Free_Act(act);
 }
 
 /***********
@@ -1014,7 +1068,9 @@ MOVI_HDR *movi_hdr;
 }
 
 
+#define MOVI_BLOCK_DEC(x,y,imagex) { x += 4; if (x>=imagex) { x=0; y -= 4; } }
 #define MOVI_BLOCK_INC(x,y,imagex) { x += 4; if (x>=imagex) { x=0; y += 4; } }
+#define MOVI_GET_8(data,dptr) {data = *dptr++; }
 #define MOVI_GET_16(data,dptr) {data = *dptr++ << 8; data |= *dptr++; }
 
 #define MVC1_COLOR2(dp,p0,c0,c1,c2,c3,c4,c5,c6,c7,mflag,map,chdr,CAST) \
@@ -1032,6 +1088,23 @@ MOVI_HDR *movi_hdr;
   p0 = *dp++ << 8; p0 |= *dp++; c6 = (CAST)MOVI_Get_Color(p0,mflag,map,chdr); \
   p0 = *dp++ << 8; p0 |= *dp++; c7 = (CAST)MOVI_Get_Color(p0,mflag,map,chdr); }
 
+#define MVC1_BLK1(ip,c0) { \
+  *ip++ = c0;			\
+  *ip++ = c0;			\
+  *ip++ = c0;			\
+  *ip   = c0; ip += row_inc;	\
+  *ip++ = c0;			\
+  *ip++ = c0;			\
+  *ip++ = c0;			\
+  *ip   = c0; ip += row_inc;	\
+  *ip++ = c0;			\
+  *ip++ = c0;			\
+  *ip++ = c0;			\
+  *ip   = c0; ip += row_inc;	\
+  *ip++ = c0;			\
+  *ip++ = c0;			\
+  *ip++ = c0;			\
+  *ip   = c0;			}
 
 #define MVC1_BLK8(ip,mask,c0,c1,c2,c3,c4,c5,c6,c7) { \
   *ip++ = (mask & 0x0001)?(c0):(c1);			\
@@ -1171,6 +1244,8 @@ XA_DEC_INFO *dec_info;  /* Decoder Info Header */
     { xaUBYTE *ip = (xaUBYTE *)(image + 3 * (y * imagex + x) );
       xaUBYTE r[8],g[8],b[8];	xaULONG mask,p0;
       MOVI_GET_16(mask,dptr); MOVI_GET_16(p0,dptr);
+
+
       if (p0 & 0x8000)	{ MVC1_RGB8(dptr,p0,r,g,b); dsize -= 18; }
       else		{ MVC1_RGB2(dptr,p0,r,g,b); dsize -=  6; }
       MVC1_RGB_BLK8(ip,mask,r,g,b);
@@ -1305,7 +1380,7 @@ XA_DEC_INFO *dec_info;  /* Decoder Info Header */
   { while( (dsize > 0) && (y < imagey))
     { xaUSHORT *ip = (xaUSHORT *)(image + ((y * imagex + x)<<1) );
       xaUSHORT c0,c1,c2,c3,c4,c5,c6,c7;
-      xaULONG mask,p0;		dsize -= 6;
+      xaULONG mask,p0;
       MOVI_GET_16(mask,dptr); MOVI_GET_16(p0,dptr);	dsize -= 6;
       if (p0 & 0x8000)  { dsize -= 12; 
 	  MVC1_COLOR8(dptr,p0,c0,c1,c2,c3,c4,c5,c6,c7,mflag,map,chdr,xaUSHORT); }
@@ -1328,13 +1403,56 @@ XA_DEC_INFO *dec_info;  /* Decoder Info Header */
   xaULONG mflag = dec_info->map_flag;        xaULONG *map = dec_info->map;
   xaULONG special = dec_info->special;
   XA_CHDR *chdr = dec_info->chdr;
-  xaULONG row_inc;
-  xaLONG x = 0, y = 0;
+  xaLONG row_inc;
   xaUBYTE *dptr = delta;
   xaULONG dither = 0;
+  xaLONG x = 0, y = 0;
+
+  xaUBYTE *mbm = 0;
+  xaULONG mbmsize = 0;
+  xaULONG xtiles, ytiles;
+  xaULONG i, nmap;
+  xaULONG lmap[256];
+
 
   dec_info->xs = dec_info->ys = 0; dec_info->xe = imagex; dec_info->ye = imagey;
-  if (dec_info->skip_flag > 0) return(ACT_DLTA_DROP);
+
+/*
+  if (dec_info->skip_flag > 0) 
+*/
+
+ 
+ MOVI_GET_16(x,dptr);
+ MOVI_GET_16(y,dptr);
+ dsize -= 4;
+ xtiles = x/4;
+ ytiles = y/4;
+
+
+ if  ( *dptr++ != 0)
+ {
+   fprintf(stderr," bitmap is present \n");
+   mbmsize = xtiles * ytiles;
+   mbmsize = 1+((mbmsize-1)/8);
+   mbm = dptr;
+   dptr += mbmsize;
+   dsize -= 1 + mbmsize;
+ }
+ else
+ {
+   mbm = 0;
+ }
+	/* Is There a Colormap, if so, read it in */
+ nmap = *dptr++;	dsize--;
+ for(i=0; i<nmap; i++)
+ { xaULONG r, g, b;
+   r = (xaULONG)(*dptr++);
+   g = (xaULONG)(*dptr++);
+   b = (xaULONG)(*dptr++);
+   lmap[i] = (xaULONG)XA_RGB24_To_CLR32(r,g,b,mflag,map,chdr);
+ }
+ dsize -= (3 * nmap);
+
 
   if (chdr) { if (chdr->new_chdr) chdr = chdr->new_chdr; }
 	/* POD rethink this */
@@ -1351,29 +1469,219 @@ XA_DEC_INFO *dec_info;  /* Decoder Info Header */
      ) dither = 1;
 
   
+/*
   row_inc = (special)?((3*(imagex-4))+1):(imagex - 3);
+*/
+  row_inc = -(imagex + 3);
 
-  if (special)
-  { fprintf(stderr,"NOT YET\n"); }
-  else if (dither)
-  { fprintf(stderr,"NOT YET\n"); }
-  else if ((x11_bytes_pixel == 1) || (mflag == xaFALSE))
-  { while( (dsize > 0) && (y < imagey))
+  x = 0;
+  y = imagey - 1;
+
+  if ((x11_bytes_pixel == 1) || (mflag == xaFALSE))
+  { while( (dsize > 0) && (y >= 0))
     { xaUBYTE *ip = (xaUBYTE *)(image + (y * imagex + x) );
       xaUBYTE c0,c1,c2,c3,c4,c5,c6,c7;
-      xaULONG mask,p0;
-      MOVI_GET_16(mask,dptr); MOVI_GET_16(p0,dptr);	dsize -= 6;
-      if (p0 & 0x8000)  { dsize -= 12; 
-	  MVC1_COLOR8(dptr,p0,c0,c1,c2,c3,c4,c5,c6,c7,mflag,map,chdr,xaUBYTE); }
-      else MVC1_COLOR2(dptr,p0,c0,c1,c2,c3,c4,c5,c6,c7,mflag,map,chdr,xaUBYTE)
-      MVC1_BLK8(ip,mask,c0,c1,c2,c3,c4,c5,c6,c7);
-      MOVI_BLOCK_INC(x,y,imagex);
-    }
-  }
+      xaULONG mask,p0,p1;
+      MOVI_GET_8(p0,dptr);
+      if (p0 & 0x80)
+      { xaUBYTE r,g,b;
+        if (p0 & 0x40)    /* 6 bit Luma */
+	{ p0 &= 0x3f;
+	  r = ((p0 << 2) | (p0 >> 4));
+	  c0 = (xaUBYTE)XA_RGB24_To_CLR32(r,r,r,mflag,map,chdr);
+	  MVC1_BLK1(ip,c0);
+	  MOVI_BLOCK_DEC(x,y,imagex);
+	  dsize -= 1;
+	}
+	else	/* 6 bit Blue, 8 bit Green 8 bit Red */
+	{ p0 &= 0x3f;
+	  b = ((p0 << 2) | (p0 >> 4));
+	  MOVI_GET_8(g,dptr);
+	  MOVI_GET_8(r,dptr);
+	  c0 = (xaUBYTE)XA_RGB24_To_CLR32(r,g,b,mflag,map,chdr);
+	  MVC1_BLK1(ip,c0);
+	  MOVI_BLOCK_DEC(x,y,imagex);
+	  dsize -= 3;
+	}
+      }
+      else
+      {
+        MOVI_GET_8(p1,dptr);
+        if (p1 & 0x80)
+        {
+	  if ((p0 & 0x7f) == (p1 & 0x7f))
+	  {
+	    c0 = lmap[ p0 & 0x7f ];
+	    MVC1_BLK1(ip,c0); 
+	    MOVI_BLOCK_DEC(x,y,imagex);
+	    dsize -= 2;
+	  }
+	  else
+	  { /* NOTE: p0 -> odds and p1 -> evens - opposite MVC1 */
+	    c0 = c2 = c4 = c6 = lmap[ p0 & 0x7f ];
+	    c1 = c3 = c5 = c7 = lmap[ p1 & 0x7f ];
+	    mask = *dptr++;  mask |= *dptr++ << 8;
+	    MVC1_BLK8(ip,mask,c1,c0,c3,c2,c5,c4,c7,c6);
+	    MOVI_BLOCK_DEC(x,y,imagex);
+	    dsize -= 4;
+	  }
+        }
+        else
+        {
+	  c0 = lmap[ p0 & 0x7f ];
+	  c1 = lmap[ p1 & 0x7f ];
+	  c2 = lmap[ (*dptr++  & 0x7f )];
+	  c3 = lmap[ (*dptr++  & 0x7f )];
+	  c4 = lmap[ (*dptr++  & 0x7f )];
+	  c5 = lmap[ (*dptr++  & 0x7f )];
+	  c6 = lmap[ (*dptr++  & 0x7f )];
+	  c7 = lmap[ (*dptr++  & 0x7f )];
+	  mask = *dptr++;  mask |= *dptr++ << 8;
+	  MVC1_BLK8(ip,mask,c1,c0,c3,c2,c5,c4,c7,c6);
+	  MOVI_BLOCK_DEC(x,y,imagex);
+	  dsize -= 10;
+        }
+      } /* end of mode 3/4 */
+    } /* while y/dsize */
+  } /* end depth 16 */
+  else if (x11_bytes_pixel == 2)
+  { while( (dsize > 0) && (y >= 0))
+    { xaUSHORT *ip = (xaUSHORT *)(image + ((y * imagex + x)<<1) );
+      xaUSHORT c0,c1,c2,c3,c4,c5,c6,c7;
+      xaULONG mask,p0,p1;
+      MOVI_GET_8(p0,dptr);
+      if (p0 & 0x80)
+      { xaUBYTE r,g,b;
+        if (p0 & 0x40)    /* 6 bit Luma */
+	{ p0 &= 0x3f;
+	  r = ((p0 << 2) | (p0 >> 4));
+	  c0 = (xaUSHORT)XA_RGB24_To_CLR32(r,r,r,mflag,map,chdr);
+	  MVC1_BLK1(ip,c0);
+	  MOVI_BLOCK_DEC(x,y,imagex);
+	  dsize -= 1;
+	}
+	else	/* 6 bit Blue, 8 bit Green 8 bit Red */
+	{ p0 &= 0x3f;
+	  b = ((p0 << 2) | (p0 >> 4));
+	  MOVI_GET_8(g,dptr);
+	  MOVI_GET_8(r,dptr);
+	  c0 = (xaUSHORT)XA_RGB24_To_CLR32(r,g,b,mflag,map,chdr);
+	  MVC1_BLK1(ip,c0);
+	  MOVI_BLOCK_DEC(x,y,imagex);
+	  dsize -= 3;
+	}
+      }
+      else
+      {
+        MOVI_GET_8(p1,dptr);
+        if (p1 & 0x80)
+        {
+	  if ((p0 & 0x7f) == (p1 & 0x7f))
+	  {
+	    c0 = lmap[ p0 & 0x7f ];
+	    MVC1_BLK1(ip,c0); 
+	    MOVI_BLOCK_DEC(x,y,imagex);
+	    dsize -= 2;
+	  }
+	  else
+	  { /* NOTE: p0 -> odds and p1 -> evens - opposite MVC1 */
+	    c0 = c2 = c4 = c6 = lmap[ p0 & 0x7f ];
+	    c1 = c3 = c5 = c7 = lmap[ p1 & 0x7f ];
+	    mask = *dptr++;  mask |= *dptr++ << 8;
+	    MVC1_BLK8(ip,mask,c1,c0,c3,c2,c5,c4,c7,c6);
+	    MOVI_BLOCK_DEC(x,y,imagex);
+	    dsize -= 4;
+	  }
+        }
+        else
+        {
+	  c0 = lmap[ p0 & 0x7f ];
+	  c1 = lmap[ p1 & 0x7f ];
+	  c2 = lmap[ (*dptr++  & 0x7f )];
+	  c3 = lmap[ (*dptr++  & 0x7f )];
+	  c4 = lmap[ (*dptr++  & 0x7f )];
+	  c5 = lmap[ (*dptr++  & 0x7f )];
+	  c6 = lmap[ (*dptr++  & 0x7f )];
+	  c7 = lmap[ (*dptr++  & 0x7f )];
+	  mask = *dptr++;  mask |= *dptr++ << 8;
+	  MVC1_BLK8(ip,mask,c1,c0,c3,c2,c5,c4,c7,c6);
+	  MOVI_BLOCK_DEC(x,y,imagex);
+	  dsize -= 10;
+        }
+      } /* end of mode 3/4 */
+    } /* while y/dsize */
+  } /* end depth 16 */
   else if (x11_bytes_pixel == 4)
-  { fprintf(stderr,"NOT YET\n"); }
-  else /* if (x11_bytes_pixel == 2) */
-  { fprintf(stderr,"NOT YET\n"); }
+  { while( (dsize > 0) && (y >= 0))
+    { xaULONG *ip = (xaULONG *)(image + ((y * imagex + x)<<2) );
+      xaULONG c0,c1,c2,c3,c4,c5,c6,c7;
+      xaULONG mask,p0,p1;
+      MOVI_GET_8(p0,dptr);
+      if (p0 & 0x80)
+      { xaUBYTE r,g,b;
+        if (p0 & 0x40)    /* 6 bit Luma */
+	{ p0 &= 0x3f;
+	  r = ((p0 << 2) | (p0 >> 4));
+	  c0 = (xaULONG)XA_RGB24_To_CLR32(r,r,r,mflag,map,chdr);
+	  MVC1_BLK1(ip,c0);
+	  MOVI_BLOCK_DEC(x,y,imagex);
+	  dsize -= 1;
+	}
+	else	/* 6 bit Blue, 8 bit Green 8 bit Red */
+	{ p0 &= 0x3f;
+	  b = ((p0 << 2) | (p0 >> 4));
+	  MOVI_GET_8(g,dptr);
+	  MOVI_GET_8(r,dptr);
+	  c0 = (xaULONG)XA_RGB24_To_CLR32(r,g,b,mflag,map,chdr);
+	  MVC1_BLK1(ip,c0);
+	  MOVI_BLOCK_DEC(x,y,imagex);
+	  dsize -= 3;
+	}
+      }
+      else
+      {
+        MOVI_GET_8(p1,dptr);
+        if (p1 & 0x80)
+        {
+	  if ((p0 & 0x7f) == (p1 & 0x7f))
+	  {
+	    c0 = lmap[ p0 & 0x7f ];
+	    MVC1_BLK1(ip,c0); 
+	    MOVI_BLOCK_DEC(x,y,imagex);
+	    dsize -= 2;
+	  }
+	  else
+	  { /* NOTE: p0 -> odds and p1 -> evens - opposite MVC1 */
+	    c0 = c2 = c4 = c6 = lmap[ p0 & 0x7f ];
+	    c1 = c3 = c5 = c7 = lmap[ p1 & 0x7f ];
+	    mask = *dptr++;  mask |= *dptr++ << 8;
+	    MVC1_BLK8(ip,mask,c1,c0,c3,c2,c5,c4,c7,c6);
+	    MOVI_BLOCK_DEC(x,y,imagex);
+	    dsize -= 4;
+	  }
+        }
+        else
+        {
+	  c0 = lmap[ p0 & 0x7f ];
+	  c1 = lmap[ p1 & 0x7f ];
+	  c2 = lmap[ (*dptr++  & 0x7f )];
+	  c3 = lmap[ (*dptr++  & 0x7f )];
+	  c4 = lmap[ (*dptr++  & 0x7f )];
+	  c5 = lmap[ (*dptr++  & 0x7f )];
+	  c6 = lmap[ (*dptr++  & 0x7f )];
+	  c7 = lmap[ (*dptr++  & 0x7f )];
+	  mask = *dptr++;  mask |= *dptr++ << 8;
+	  MVC1_BLK8(ip,mask,c1,c0,c3,c2,c5,c4,c7,c6);
+	  MOVI_BLOCK_DEC(x,y,imagex);
+	  dsize -= 10;
+        }
+      } /* end of mode 3/4 */
+    } /* while y/dsize */
+  } /* end depth 32 */
+  else
+  {
+    return(ACT_DLTA_DROP);
+  }
 
   if (mflag) return(ACT_DLTA_MAPD);
   else return(ACT_DLTA_NORM);
@@ -1419,3 +1727,88 @@ XA_CHDR *chdr;
   }
   return(clr);
 }
+
+xaULONG MOVI_Decode_RGB32(image,delta,tdsize,dec_info)
+xaUBYTE *image;         /* Image Buffer. */
+xaUBYTE *delta;         /* delta data. */
+xaULONG tdsize;          /* delta size */
+XA_DEC_INFO *dec_info;  /* Decoder Info Header */
+{ xaULONG imagex = dec_info->imagex;    xaULONG imagey = dec_info->imagey;
+  xaULONG mflag = dec_info->map_flag;        xaULONG *map = dec_info->map;
+  xaULONG special = dec_info->special;
+  XA_CHDR *chdr = dec_info->chdr;
+  xaULONG row_inc;
+  xaLONG x = 0, y = 0;
+  xaUBYTE *dp = delta;
+  xaULONG dither = 0;
+  xaLONG dsize = tdsize;
+
+  dec_info->xs = dec_info->ys = 0;
+  dec_info->xe = imagex;
+  dec_info->ye = imagey;
+	/* Indicate we can drop these frames */
+  if (dec_info->skip_flag > 0) return(ACT_DLTA_DROP);
+
+  if (chdr) { if (chdr->new_chdr) chdr = chdr->new_chdr; }
+
+/*
+  if (   (!special) && (x11_bytes_pixel==1)
+      && (chdr) && (x11_display_type == XA_PSEUDOCOLOR)
+      && (cmap_color_func != 4)
+      && (cmap_true_to_332 == xaTRUE) && (x11_cmap_size == 256)
+      && (xa_dither_flag==xaTRUE)
+     ) dither = 1;
+*/
+
+  row_inc = (special)?(3*imagex):imagex;
+
+  if (special)
+  { while( (dsize > 0) && (y < imagey))
+    { xaUBYTE *ip = (xaUBYTE *)(image + (3 * y * imagex)); y++;
+      x = imagex;
+      while(x--) {xaUBYTE r,g,b; dp++; r = *dp++; g = *dp++; b = *dp++;
+	*ip++ = r; *ip++ = g; *ip++ = b; }
+      dsize -= (imagex << 2);
+    }
+  }
+  else if ((x11_bytes_pixel == 1) || (mflag == xaFALSE))
+  { while( (dsize > 0) && (y < imagey))
+    { xaUBYTE *ip = (xaUBYTE *)(image + y * imagex);	y++;
+      x = imagex;
+      while(x--)
+      { xaULONG r,g,b; 
+	dp++; r = (xaULONG)*dp++; g = (xaULONG)*dp++; b = (xaULONG)*dp++;
+	*ip++ = (xaUBYTE)XA_RGB24_To_CLR32(r,g,b,mflag,map,chdr);
+      }
+      dsize -= (imagex << 2);
+    }
+  } 
+  else if (x11_bytes_pixel == 4)
+  { 
+    while( (dsize > 0) && (y < imagey))
+    { xaULONG *ip = (xaULONG *)(image + ((y * imagex + x)<<2) );  y++;
+      x = imagex;
+      while(x--)
+      { xaULONG r,g,b; 
+	dp++; r = (xaULONG)*dp++; g = (xaULONG)*dp++; b = (xaULONG)*dp++;
+	*ip++ = (xaULONG)XA_RGB24_To_CLR32(r,g,b,mflag,map,chdr);
+      }
+      dsize -= (imagex << 2);
+    }
+  }
+  else /* if (x11_bytes_pixel == 2) */
+  { while( (dsize > 0) && (y < imagey))
+    { xaUSHORT *ip = (xaUSHORT *)(image + ((y * imagex + x)<<1) );  y++;
+      x = imagex;
+      while(x--)
+      { xaULONG r,g,b; 
+	dp++; r = (xaULONG)*dp++; g = (xaULONG)*dp++; b = (xaULONG)*dp++;
+	*ip++ = (xaUSHORT)XA_RGB24_To_CLR32(r,g,b,mflag,map,chdr);
+      }
+      dsize -= (imagex << 2);
+    }
+  } 
+  if (mflag) return(ACT_DLTA_MAPD);
+  else return(ACT_DLTA_NORM);
+}
+
