@@ -2,15 +2,15 @@
 /*
  * xa_mpg.c
  *
- * Copyright (C) 1995,1996,1997 by Mark Podlipec.
+ * Copyright (C) 1995-1997,1998 by Mark Podlipec.
  * All rights reserved.
  *
- * This software may be freely copied, modified and redistributed without
- * fee for non-commerical purposes provided that this copyright notice is
- * preserved intact on all copies and modified copies.
+ * This software may be freely used, copied and redistributed without
+ * fee for non-commerical purposes provided that this copyright
+ * notice is preserved intact on all copies.
  *
  * There is no warranty or other guarantee of fitness of this software.
- * It is provided solely "as is". The author(s) disclaim(s) all
+ * It is provided solely "as is". The author disclaims all
  * responsibility and liability with respect to this software's usage
  * or its effect upon hardware or computer systems.
  *
@@ -392,16 +392,19 @@ XA_ANIM_HDR *anim_hdr;
 	  break;
       }
     }
-    else if (mpg_start_code == MPG_UNK_BE)
-		{ DEBUG_LEVEL1 fprintf(stderr,"MPG: BE code  - ignored\n"); }
-    else if (mpg_start_code == MPG_PACK_START)
-		{ DEBUG_LEVEL1 fprintf(stderr,"MPG: BA code  - ignored\n"); }
-    else if (mpg_start_code == MPG_SYS_START)
-		{ DEBUG_LEVEL1 fprintf(stderr,"MPG: BB code  - ignored\n"); }
-    else if (mpg_start_code == MPG_UNK_BC)
-		{ DEBUG_LEVEL1 fprintf(stderr,"MPG: BC code  - ignored\n"); }
-    else if (mpg_start_code == MPG_UNK_C0)
-		{ DEBUG_LEVEL1 fprintf(stderr,"MPG: C0 code  - ignored\n"); }
+	/* For now skip system and most packet start codes */
+    else if ( (mpg_start_code >= 0xb9)
+		&& (mpg_start_code <= 0xdf) )
+    { 
+      DEBUG_LEVEL1 fprintf(stderr,
+		"MPG: unsupported code %02x - ignored\n", mpg_start_code);
+    }
+    else if ( (mpg_start_code >= 0xe1)
+		&& (mpg_start_code <= 0xff) )
+    { 
+      DEBUG_LEVEL1 fprintf(stderr,
+		"MPG: unsupported code %02x - ignored\n", mpg_start_code);
+    }
     else if (mpg_start_code == MPG_GOP_START)	mpg_read_GOP_HDR();
     else if (mpg_start_code == MPG_SEQ_END)	break; /* END??? */
     else if (mpg_start_code == MPG_USR_START)	
