@@ -2,15 +2,15 @@
 /*
  * xa_qt.c
  *
- * Copyright (C) 1993,1994,1995,1996,1997 by Mark Podlipec.
+ * Copyright (C) 1993-1997,1998 by Mark Podlipec.
  * All rights reserved.
  *
- * This software may be freely copied, modified and redistributed without
- * fee for non-commerical purposes provided that this copyright notice is
- * preserved intact on all copies and modified copies.
+ * This software may be freely used, copied and redistributed without
+ * fee for non-commerical purposes provided that this copyright
+ * notice is preserved intact on all copies.
  *
  * There is no warranty or other guarantee of fitness of this software.
- * It is provided solely "as is". The author(s) disclaim(s) all
+ * It is provided solely "as is". The author disclaims all
  * responsibility and liability with respect to this software's usage
  * or its effect upon hardware or computer systems.
  *
@@ -174,7 +174,7 @@ QT_HDLR_HDR qt_hdlr_hdr;
 
 char qt_rfilename[256];
 char qt_dfilename[256];
-static xaULONG qt_video_flag;
+static xaULONG qt_video_flag, qt_audio_flag;
 static xaULONG qt_data_flag;
 static xaULONG qt_v_flag, qt_s_flag;
 static xaULONG qt_moov_flag;
@@ -270,6 +270,8 @@ xaULONG qt_stgs_num;
 
 xaULONG qt_has_ctab;
 
+xaULONG qt_saw_audio, qt_saw_video;
+
 /* main() */
 xaULONG QT_Read_File(fname,anim_hdr,audio_attempt)
 char *fname;
@@ -288,6 +290,8 @@ xaULONG audio_attempt;  /* xaTRUE is audio is to be attempted */
   qt->compression	= QT_CODEC_UNK;
 
 
+  qt_saw_audio	= xaFALSE;
+  qt_saw_video	= xaFALSE;
   qt_has_audio	= xaFALSE;
   qt_has_video	= xaFALSE;
   qt_has_ctab	= xaFALSE;
@@ -300,6 +304,7 @@ xaULONG audio_attempt;  /* xaTRUE is audio is to be attempted */
   qtv_codec_num 	= qts_codec_num = 0;
   qt_data_flag = xaFALSE;
   qt_video_flag		= 0;
+  qt_audio_flag		= 0;
   qt_v_flag		= qt_s_flag = 0;
   qt_moov_flag = xaFALSE;
 
@@ -346,12 +351,14 @@ xaULONG audio_attempt;  /* xaTRUE is audio is to be attempted */
     XA_Free_Anim_Setup(qt);
     return(xaFALSE);
   }
+
   if (qt_data_flag == xaFALSE) 
   { /* mdat was not in .rscr file need to open .data file */
     fclose(fin); /* close .rscr file */
     if (qt_dfilename[0] == 0)
-    { fprintf(stdout,"QT_Data: No data in %s file. Can't find .data file.\n",
+    { fprintf(stdout,"QT_Data: No data in %s file and no *.data file.\n",
 		qt_rfilename);
+      fprintf(stdout,"         Some Quicktimes do not contain info playable by XAnim.\n");
       return(xaFALSE);
     }
     if ( (fin=fopen(qt_dfilename,XA_OPEN_MODE)) == 0) 
@@ -362,29 +369,63 @@ xaULONG audio_attempt;  /* xaTRUE is audio is to be attempted */
 
 DEBUG_LEVEL1 fprintf(stdout,"reading data\n");
 
-  if (qtv_samp_sizes)	qt_has_video = QT_Read_Video_Data(qt,fin,anim_hdr);
-  if ((qts_samp_sizes) && (qt_audio_attempt==xaTRUE))
-			qt_has_audio = QT_Read_Audio_Data(qt,fin,anim_hdr);
+  if (qtv_samp_sizes && qt_video_flag)
+		qt_has_video = QT_Read_Video_Data(qt,fin,anim_hdr);
+  if (qts_samp_sizes && qt_audio_flag && (qt_audio_attempt==xaTRUE))
+		qt_has_audio = QT_Read_Audio_Data(qt,fin,anim_hdr);
   fclose(fin);
 
   if ((qt_has_video == xaFALSE) || (qt_frame_cnt == 0))
   {
+
     if (qt_has_audio == xaFALSE)  /* no video and no audio */
-    { if (qt_moov_flag==xaTRUE)
-      {
-        if (qt_audio_attempt == xaFALSE)
-        {
-           fprintf(stdout,"QT: no video to play. possibly truncated.\n");
-        }
-        else fprintf(stdout,"QT: no video/audio to play. possibly truncated.\n");
+    { 
+
+      if ((qt_saw_video == xaTRUE) && (qt_saw_audio == xaTRUE))
+      { fprintf(stdout,
+	   "  Notice: Video and Audio are present, but not yet supported.\n");
+      }
+      else if ((qt_saw_video == xaTRUE) && (qt_saw_audio == xaFALSE))
+      { fprintf(stdout,
+	   "  Notice: Video is present, but not yet supported.\n");
+      }
+      else if ((qt_saw_video == xaFALSE) && (qt_saw_audio == xaTRUE))
+      { fprintf(stdout,
+	   "  Notice: Audio is present, but not yet supported.\n");
       }
       else
-	fprintf(stdout,"QT: file possibly truncated or missing .rsrc info.\n");
+      {		 /* At least we saw the moov header */
+        if (qt_moov_flag == xaTRUE)
+	{
+		/* No Video and we weren't trying for audio */
+          if (qt_audio_attempt == xaFALSE)
+          {
+             fprintf(stdout,"QT: no video to play. possibly truncated.\n");
+          }
+          else
+	  {
+		/* No Video and No Audio */
+	    fprintf(stdout,"QT: no video/audio to play. possibly truncated.\n");
+	  }
+        }
+        else  /* either truncated or missing .rsrc file */
+	{
+	  fprintf(stdout,
+		"QT: file possibly truncated or missing .rsrc info.\n");
+        }
+      }
       return(xaFALSE);
     }
 
+    if (qt_saw_video == xaTRUE)
+    { fprintf(stdout,
+	   "  Notice: Video is present, but not yet supported.\n");
+    }
+
+/* NO LONGER???
     if (qtv_samp_sizes)
 	fprintf(stdout,"QT Notice: No supported Video frames - treating as audio only file\n");
+*/
 
     anim_hdr->total_time = (qt_mvhdr.duration * 1000) / (qt_mvhdr.timescale);
   }
@@ -579,6 +620,12 @@ FILE *fin;
     len = UTIL_Get_MSB_Long(fin);
     id  = UTIL_Get_MSB_Long(fin);
 
+/* if (xa_verbose) */
+DEBUG_LEVEL1 
+	fprintf(stdout,"%c%c%c%c %04x len = %x file_len =  %x\n",
+	(char)(id >> 24),(char)((id>>16)&0xff),
+	(char)((id>>8)&0xff),(char)(id&0xff),id,len,file_len);
+
     if ( (len == 0) && (id == QT_mdat) )
     {
       fprintf(stdout,"QT: mdat len is 0. You also need a .rsrc file.\n");
@@ -587,12 +634,13 @@ FILE *fin;
     if (len < 8) { file_len = 0; continue; } /* just bad - finish this */
     if (file_len == 1)
     {
-      if (id == QT_moov) file_len = len + 8;
-      else file_len = len + 8;
+      if (id == QT_moov) file_len = len;
+      else file_len = len;
     }
-    DEBUG_LEVEL2 fprintf(stdout,"%c%c%c%c %04x len = %x file_len =  %x\n",
-	(char)(id >> 24),(char)((id>>16)&0xff),
-	(char)((id>>8)&0xff),(char)(id&0xff),id,len,file_len);
+
+/* if (xa_verbose) */
+DEBUG_LEVEL1 
+    fprintf(stdout,"  len = %x file_len = %x\n",len,file_len);
 
     switch(id)
     {
@@ -643,22 +691,25 @@ FILE *fin;
     /*--------------DATA CHUNKS-------------*/
       case QT_mdat:  /* data is included in .rsrc - assumed flatness */
 	fseek(fin,(len-8),1); /* skip over it for now */
+	file_len -= len;
 	qt_data_flag = xaTRUE;
 	break;
       case QT_stsd:
-	qt_video_flag = 0;
-	if (qt_v_flag) 
-	{
-	  if (QT_Read_Video_STSD(anim_hdr,qt,fin) == xaFALSE) return(xaFALSE);
+	if (qt_v_flag)
+	{ qt_saw_video = xaTRUE;
+	  QT_Read_Video_STSD(anim_hdr,qt,fin,(len-8));
+	}
+	else if (qt_s_flag)
+	{ qt_saw_audio = xaTRUE;
+	  QT_Read_Audio_STSD(fin,(len-8));
 	}
-	else if (qt_s_flag) QT_Read_Audio_STSD(fin);
         else fseek(fin,(len-8),1);
 	file_len -= len;
 	break;
       case QT_stts:
 	if (qt_v_flag) 
 		QT_Read_STTS(fin,(len-8),&qtv_t2samp_num,&qtv_t2samps);
-/*POD NOTE: AUDIO don't need? probably, just haven't been bit by it yet.
+/*POD NOTE: AUDIO doesn't need? probably, just haven't been bit by it yet.
 	else if (qt_s_flag) 
 		QT_Read_STTS(fin,(len-8),&qts_t2samp_num,&qts_t2samps);
 */
@@ -761,11 +812,25 @@ if (xa_verbose) fprintf(stdout,"QT: has ctab\n");
 	  qt_has_ctab = xaTRUE;
 	}
 	break;
+    /*--------------Indicated We Don't Support this File -----------*/
+      case QT_rmra:
+	fprintf(stdout,"NOTE: RMRA chunk is not yet supported.\n");
+        fseek(fin,(len-8),1);  /* skip over */
+	file_len -= len;
+	break;
+      case QT_cmov:
+	fprintf(stdout,
+	     "NOTE: This Quicktime file uses compressed headers. XAnim is not\n");
+	fprintf(stdout,
+	     "      yet able to support this Apple proprietary feature.\n");
+	return(xaFALSE);
+	break;
     /*--------------IGNORED FOR NOW---------*/
       case QT_gmhd:
       case QT_text:
       case QT_clip:
       case QT_skip:
+      case QT_free:
       case QT_udta:
       case QT_dinf:
         fseek(fin,(len-8),1);  /* skip over */
@@ -776,14 +841,20 @@ if (xa_verbose) fprintf(stdout,"QT: has ctab\n");
 	if ( !feof(fin) && (len <= file_len) )
 	{
 	  xaLONG i;
+
+DEBUG_LEVEL1
+{
 	  QT_Print_ID(stdout,id,1);
-	  fprintf(stdout," len = %x\n",len);
+	  fprintf(stdout," len = %x file_len %x\n",len,file_len);
+}
 	  i = (xaLONG)(len) - 8;
+		/* read and check eof */
 	  while(i > 0) { i--; getc(fin); if (feof(fin)) i = 0; }
 	}
 	file_len -= len;
 	break;
     } /* end of switch */
+
     if ( feof(fin) ) 
     {
       file_len = 0;
@@ -792,6 +863,16 @@ if (xa_verbose) fprintf(stdout,"QT: has ctab\n");
 	return(xaFALSE);
       }
     }
+    else if (file_len <= 0)
+    {
+	/* interesting dilemma */
+      if (   (qt_data_flag == xaFALSE)
+	  || (qt_moov_flag == xaFALSE))
+      {
+	/** START OVER **/
+	file_len = 1;
+      }
+    }
   } /* end of while */
   return(xaTRUE);
 }
@@ -880,8 +961,8 @@ QT_TKHDR *qt_tkhdr;
 }
 
 
-/* PODPOD */
-/* implement different between start_offset and init_duration */
+/* POD TODO */
+/* implement difference between start_offset and init_duration */
 void QT_Read_ELST(fin,qt_start_offset,qt_init_duration)
 FILE *fin;
 xaULONG  *qt_start_offset;
@@ -962,13 +1043,15 @@ QT_HDLR_HDR *qt_hdlr_hdr;
  * Read and Parse Video Codecs
  *
  **********/
-xaULONG QT_Read_Video_STSD(anim_hdr,qt,fin)
-XA_ANIM_HDR *anim_hdr;
-XA_ANIM_SETUP *qt;
-FILE *fin;
+xaULONG QT_Read_Video_STSD(anim_hdr,qt,fin,clen)
+XA_ANIM_HDR	*anim_hdr;
+XA_ANIM_SETUP	*qt;
+FILE		*fin;
+xaLONG		clen;
 { xaULONG i,version,num,cur,sup;
   version = UTIL_Get_MSB_Long(fin);
   num = UTIL_Get_MSB_Long(fin);
+  clen -= 8;
   DEBUG_LEVEL2 fprintf(stdout,"     ver = %x  num = %x\n", version,num);
   if (qtv_codecs == 0)
   { qtv_codec_num = num;
@@ -989,10 +1072,15 @@ FILE *fin;
   sup = 0;
   for(i=0; i < num; i++)
   {
-    sup |= QT_Read_Video_Codec_HDR(anim_hdr,qt, &qtv_codecs[cur], fin ); 
+    sup |= QT_Read_Video_Codec_HDR(anim_hdr,qt, &qtv_codecs[cur], fin, &clen ); 
 DEBUG_LEVEL1 fprintf(stdout,"CODEC %d) sup=%d\n",i,sup);
     cur++;
   }
+  if (clen)
+  { DEBUG_LEVEL1 fprintf(stdout,"QT Video STSD len mismatch %x\n",clen); 
+    fseek(fin,clen,1);
+  }
+
   if (sup == 0) return(xaFALSE);
   qt_video_flag = 1; 
   if ( (qt->pic==0) && (xa_buffer_flag == xaTRUE))
@@ -1007,11 +1095,12 @@ DEBUG_LEVEL1 fprintf(stdout,"CODEC %d) sup=%d\n",i,sup);
 }
 
 
-xaULONG QT_Read_Video_Codec_HDR(anim_hdr,qt,c_hdr,fin)
+xaULONG QT_Read_Video_Codec_HDR(anim_hdr,qt,c_hdr,fin, clen)
 XA_ANIM_HDR *anim_hdr;
 XA_ANIM_SETUP *qt;
 QTV_CODEC_HDR *c_hdr;
 FILE *fin;
+xaLONG		*clen;
 {
   xaULONG id;
   xaLONG len,i,codec_ret;
@@ -1019,6 +1108,7 @@ FILE *fin;
   xaULONG vendor,temp_qual,spat_qual,h_res,v_res;
   
   len		= UTIL_Get_MSB_Long(fin);
+  *clen		-= len;
   id 		= UTIL_Get_MSB_Long(fin);
   unk_0		= UTIL_Get_MSB_Long(fin);
   unk_1		= UTIL_Get_MSB_Long(fin);
@@ -1048,10 +1138,12 @@ FILE *fin;
   c_hdr->chdr	= 0;
 
   if (   (qt->depth == 8) || (qt->depth == 40)
-      || (qt->depth == 4) || (qt->depth == 36) ) /* generate colormap */
+      || (qt->depth == 4) || (qt->depth == 36)
+      || (qt->depth == 2) || (qt->depth == 34) ) /* generate colormap */
   {
-    if (qt->depth & 0x04) qt->imagec = 16;
-    else qt->imagec = 256;
+    if      ((qt->depth & 0x0f) == 0x04) qt->imagec = 16;
+    else if ((qt->depth & 0x0f) == 0x02) qt->imagec = 4;
+    else				 qt->imagec = 256;
 
     if (qt->depth < 32) QT_Create_Default_Cmap(qt->cmap,qt->imagec);
     else /* grayscale */
@@ -1088,7 +1180,10 @@ DEBUG_LEVEL1 fprintf(stdout," start %x end %x cflag %x\n",start,end,cflag);
     } 
   }
 
-  while(len > 0) {fgetc(fin); len--; }
+  if (len) 
+  { fseek(fin,len,1);
+    len = 0;
+  }
 
   qt->compression = qt_codec_hdr.compression = id;
   qt_codec_hdr.x = qt->imagex;
@@ -1132,10 +1227,8 @@ DEBUG_LEVEL1 fprintf(stdout," start %x end %x cflag %x\n",start,end,cflag);
 		(char)a[3],(char)a[2],(char)a[1],(char)a[0],qt->compression);
       qt_codec_hdr.description = tmpbuf;
     }
-    if (xa_verbose) 
-		fprintf(stdout,"  Video Codec: %s",qt_codec_hdr.description);
-    else	fprintf(stdout,"QT Video Codec: %s",qt_codec_hdr.description);
-    fprintf(stdout," is unsupported by this executable.(E%x)\n",qt->depth);
+    fprintf(stdout,"  Video Codec: %s",qt_codec_hdr.description);
+    fprintf(stdout," not yet supported.(E%x)\n",qt->depth);
 /* point 'em to the readme's */
     switch(qt->compression)
     { 
@@ -1145,11 +1238,11 @@ DEBUG_LEVEL1 fprintf(stdout," start %x end %x cflag %x\n",start,end,cflag);
       case QT_IV32: 
       case QT_YVU9: 
       case QT_YUV9: 
-        fprintf(stdout,"      Please see the file \"indeo.readme\".\n");
+        fprintf(stdout,"      To support this Codec please read the file \"indeo.readme\".\n");
         break;
       case QT_cvid: 
       case QT_CVID: 
-        fprintf(stdout,"      Please see the file \"cinepak.readme\".\n");
+        fprintf(stdout,"      To support this Codec please read the file \"cinepak.readme\".\n");
         break;
     }
     return(0);
@@ -1610,6 +1703,18 @@ DEBUG_LEVEL2 fprintf(stdout,"T2S: cur-t2 %d cur-smp %d nxt-t2 %d tot t2 %d\n", c
 	t_time   += qt->vid_time;
 	t_timelo += qt->vid_timelo;
 	while(t_timelo >= 1<<24) { t_timelo -= 1<<24; t_time++; }
+
+/* Some Quicktimes have two tracks. One chock full of video and one
+ * with a single image and a duration of the entire clip.
+ * Unfortunately, by the time XAnim is done with that image, it's
+ * too late for the others.
+ *
+ * TODO: Need a routine to detect overlaps and shorten as necessary.
+ *
+ */
+/*TESTING*/
+/* if (t_time > 500) t_time = 0; */
+
         QT_Add_Frame(t_time,t_timelo,act);
         qtv_init_duration = 0;
       }
@@ -1628,7 +1733,8 @@ DEBUG_LEVEL2 fprintf(stdout,"T2S: cur-t2 %d cur-smp %d nxt-t2 %d tot t2 %d\n", c
       {
 	if (unsupported_flag == xaFALSE) /* only output single warning */
 	{
-	  fprintf(stdout,"QT: Sections of this movie use an unsupported Video Codec\n");
+	  fprintf(stdout,
+		"QT: Sections of this movie use an unsupported Video Codec\n");
 	  fprintf(stdout," and are therefore NOT viewable.\n");
 	  unsupported_flag = xaTRUE;
         }
@@ -1724,22 +1830,25 @@ DEBUG_LEVEL1 fprintf(stdout,"QT_Parse_Bin: pos %x Total %x\n",pos,total);
  * Read and Parse Audio Codecs
  *
  **********/
-void QT_Read_Audio_STSD(fin)
-FILE *fin;
+void QT_Read_Audio_STSD(fin,clen)
+FILE	*fin;
+xaLONG	clen;
 { xaULONG i,version,num,cur,sup;
-  version = UTIL_Get_MSB_Long(fin);
-  num = UTIL_Get_MSB_Long(fin);
+  version	= UTIL_Get_MSB_Long(fin);
+  num		= UTIL_Get_MSB_Long(fin);
+  clen -= 8;
   DEBUG_LEVEL2 fprintf(stdout,"     ver = %x  num = %x\n", version,num);
   if (qts_codecs == 0)
   { qts_codec_num = num;
-    qts_codecs = (QTS_CODEC_HDR *)malloc(qts_codec_num * sizeof(QTS_CODEC_HDR));
+    qts_codecs = (QTS_CODEC_HDR *)malloc(qts_codec_num 
+						* sizeof(QTS_CODEC_HDR));
     if (qts_codecs==0) TheEnd1("QT STSD: malloc err");
     cur = 0;
   }
   else
   { QTS_CODEC_HDR *tcodecs;
-    tcodecs = (QTS_CODEC_HDR *)malloc((qts_codec_num+num) * sizeof(QTS_CODEC_HDR))
-;
+    tcodecs = (QTS_CODEC_HDR *)malloc((qts_codec_num + num)
+						* sizeof(QTS_CODEC_HDR));
     if (tcodecs==0) TheEnd1("QT STSD: malloc err");
     for(i=0;i<qts_codec_num;i++) tcodecs[i] = qts_codecs[i];
     cur = qts_codec_num;
@@ -1750,27 +1859,35 @@ FILE *fin;
   sup = 0;
   for(i=0; i < num; i++)
   {
-    sup |= QT_Read_Audio_Codec_HDR( &qts_codecs[cur], fin );
+    sup |= QT_Read_Audio_Codec_HDR( &qts_codecs[cur], fin, &clen );
     cur++;
   }
   if (sup == 0)
   {
     if (qt_audio_attempt == xaTRUE) 
     {
-      fprintf(stdout,"QT Audio Codec not supported\n");
       qt_audio_attempt = xaFALSE;
     }
   }
+  else qt_audio_flag = 1; 
+
+	/** If Extra bytes, skip over them **/
+  if (clen)
+  { DEBUG_LEVEL1 fprintf(stdout,"QT Audio STSD len mismatch %x\n",clen); 
+    fseek(fin,clen,1);
+  }
 }
 
 
-xaULONG QT_Read_Audio_Codec_HDR(c_hdr,fin)
-QTS_CODEC_HDR *c_hdr;
-FILE *fin;
+xaULONG QT_Read_Audio_Codec_HDR(c_hdr,fin, clen)
+QTS_CODEC_HDR	*c_hdr;
+FILE		*fin;
+xaLONG		*clen;
 { xaULONG len;
   xaULONG ret = 1;
   len			= UTIL_Get_MSB_Long(fin);
-  c_hdr->compression	= UTIL_Get_MSB_Long(fin);
+  c_hdr->format		= UTIL_Get_MSB_Long(fin);
+  c_hdr->compression	= XA_AUDIO_INVALID;
   c_hdr->dref_id	= UTIL_Get_MSB_Long(fin);
   c_hdr->version	= UTIL_Get_MSB_Long(fin);
   c_hdr->codec_rev	= UTIL_Get_MSB_Long(fin);
@@ -1780,9 +1897,9 @@ FILE *fin;
 
 /* Some Mac Shareware program screws up when converting AVI's to QT's */
   if (   (c_hdr->bits_samp == 1)
-      && (   (c_hdr->compression == QT_twos)
-          || (c_hdr->compression == QT_raw)
-          || (c_hdr->compression == QT_raw00)
+      && (   (c_hdr->format == QT_twos)
+          || (c_hdr->format == QT_raw)
+          || (c_hdr->format == QT_raw00)
      )  )                                       c_hdr->bits_samp = 8;
 
   c_hdr->comp_id	= UTIL_Get_MSB_UShort(fin);
@@ -1790,18 +1907,53 @@ FILE *fin;
   c_hdr->samp_rate	= UTIL_Get_MSB_UShort(fin);
   c_hdr->pad		= UTIL_Get_MSB_UShort(fin);
 
-  if (xa_verbose)
+  *clen -= 36;
+
+  if (c_hdr->format == QT_twos) c_hdr->compression = XA_AUDIO_SIGNED;
+  else if (c_hdr->format == QT_raw) c_hdr->compression = XA_AUDIO_LINEAR;
+  else if (c_hdr->format == QT_raw00) c_hdr->compression = XA_AUDIO_LINEAR;
+  else if (c_hdr->format == QT_ima4) c_hdr->compression = XA_AUDIO_IMA4;
+  else if (c_hdr->format == QT_agsm) c_hdr->compression = XA_AUDIO_GSM;
+  else if (c_hdr->format == QT_ulaw)
+  {  c_hdr->compression = XA_AUDIO_ULAW;
+     c_hdr->bits_samp = 8;  /* why they list it at 16 I have no clue */
+  }
+  else if (c_hdr->format == QT_MAC3)
+  { fprintf(stdout,
+	"  Audio Codec: Apple MAC3 not yet supported.\n");
+     ret = 0;
+  }
+  else if (c_hdr->format == QT_MAC6)
+  { fprintf(stdout,
+	"  Audio Codec: Apple MAC6 not yet supported.\n");
+     ret = 0;
+  }
+  else if (c_hdr->format == QT_QDMC)
+  { fprintf(stdout,
+	"  Audio Codec: QDesign Music Codec (QDMC) not yet supported.\n");
+     ret = 0;
+  }
+  else if (c_hdr->format == QT_Qclp)
+  { fprintf(stdout,
+	"  Audio Codec: QualComm PureVoice (Qclp) not yet supported.\n");
+     ret = 0;
+  }
+  else
+  { xaULONG t = c_hdr->format;
+    fprintf(stdout,"Unknown(and unsupported) Audio Codec: %c%c%c%c(0x%x).\n",
+		((t>>24) & 0x7f), ((t>>16) & 0x7f), 
+		((t>>8) & 0x7f), (t & 0x7f),		 t);
+    ret = 0;
+  }
+
+/*** If not supported, then already output ***/
+  if (ret & xa_verbose)
   {
-    fprintf(stdout,"  Audio Codec: "); QT_Audio_Type(c_hdr->compression);
+    fprintf(stdout,"  Audio Codec: "); QT_Audio_Type(c_hdr->format);
     fprintf(stdout," Rate=%d Chans=%d Bps=%d\n",
 	c_hdr->samp_rate,c_hdr->chan_num,c_hdr->bits_samp);
   }
 
-  if (c_hdr->compression == QT_twos) c_hdr->compression =XA_AUDIO_SIGNED;
-  else if (c_hdr->compression == QT_raw) c_hdr->compression =XA_AUDIO_LINEAR;
-  else if (c_hdr->compression == QT_raw00) c_hdr->compression =XA_AUDIO_LINEAR;
-  else if (c_hdr->compression == QT_ima4) c_hdr->compression =XA_AUDIO_IMA4;
-  else ret = 0;
   if (c_hdr->bits_samp==8) c_hdr->bps = 1;
   else if (c_hdr->bits_samp==16) c_hdr->bps = 2;
   else if (c_hdr->bits_samp==32) c_hdr->bps = 4;
@@ -1810,8 +1962,12 @@ FILE *fin;
   if (c_hdr->bps > 2) ret = 0;
   if (c_hdr->chan_num > 2) ret = 0;
 
-  if (   (c_hdr->bps==2) 
-      && ((c_hdr->compression & XA_AUDIO_TYPE_MASK) != XA_AUDIO_IMA4) )
+/* This is only valid for PCM type samples */
+  if ( (c_hdr->bps==2)  &&
+	 (   ((c_hdr->compression & XA_AUDIO_TYPE_MASK) == XA_AUDIO_LINEAR)
+	  || ((c_hdr->compression & XA_AUDIO_TYPE_MASK) == XA_AUDIO_SIGNED)
+	 )
+     )
   {
     c_hdr->compression |= XA_AUDIO_BPS_2_MSK;
     c_hdr->compression |= XA_AUDIO_BIGEND_MSK; /* only has meaning >= 2 bps */
@@ -1831,8 +1987,10 @@ xaULONG type;
   { case QT_raw:	fprintf(stdout,"PCM"); break;
     case QT_raw00:	fprintf(stdout,"PCM0"); break;
     case QT_twos:	fprintf(stdout,"TWOS"); break;
+    case QT_MAC3:	fprintf(stdout,"MAC3"); break;
     case QT_MAC6:	fprintf(stdout,"MAC6"); break;
     case QT_ima4:	fprintf(stdout,"IMA4"); break;
+    case QT_ulaw:	fprintf(stdout,"uLaw"); break;
     default:		fprintf(stdout,"(%c%c%c%c)(%08x)",
 			  (char)((type>>24)&0xff), (char)((type>>16)&0xff),
 			  (char)((type>>8)&0xff),(char)((type)&0xff),type); 
@@ -1866,6 +2024,7 @@ DEBUG_LEVEL1 fprintf(stdout,"QT_Read_Audio: attempt %x co# %d\n",
   qt_audio_end	 = 1;
 
   /* Initial Silence if any. PODNOTE: Eventually Modify for Middle silence */
+  /**** TODO: detect middle silences and add NOP audio chunks ****/
   qts_init_duration += qtv_start_offset;
   if (qts_init_duration)
   { xaULONG snd_size;
@@ -1934,6 +2093,12 @@ fprintf(stdout,"SND: off %08x size %08x\n",chunk_off, size);
       blockalign = 0x44;
       sampsblock = 0x40;
     }
+    else if (qt_audio_type == XA_AUDIO_GSM_M)
+    { xaULONG numblks = num_samps / 160;
+      snd_size = numblks * 33; 
+      blockalign = 33;
+      sampsblock = 160;
+    }
     else
     { snd_size = num_samps * qt_audio_bps;
       blockalign = qt_audio_bps; /* not really valid yet */
@@ -1992,6 +2157,12 @@ xaLONG len;
   while(len > 0) {len--; getc(fin); }
 }
 
+int qt_2map[] = {
+0x93, 0x65, 0x5e,
+0xff, 0xff, 0xff,
+0xdf, 0xd0, 0xab,
+0x00, 0x00, 0x00
+};
 
 /* MAX */
 int qt_4map[] = {
@@ -2009,7 +2180,8 @@ int qt_4map[] = {
 0xdf, 0xd0, 0xab,
 0xff, 0xfb, 0xf9,
 0xe8, 0xca, 0xc5,
-0x8a, 0x7c, 0x77
+0x8a, 0x7c, 0x77,
+0x00, 0x00, 0x00
 };
 
 
@@ -2026,9 +2198,18 @@ xaULONG cnum;
 {
   xaLONG r,g,b,i;
 
-  if (cnum == 16)
+  if (cnum == 4)
   { 
-    for(i=0;i<15;i++)
+    for(i=0;i<4;i++)
+    { int d = i * 3;
+      cmap[i].red   = 0x101 * qt_2map[d];
+      cmap[i].green = 0x101 * qt_2map[d+1];
+      cmap[i].blue  = 0x101 * qt_2map[d+2];
+    }
+  }
+  else if (cnum == 16)
+  { 
+    for(i=0;i<16;i++)
     { int d = i * 3;
       cmap[i].red   = 0x101 * qt_4map[d];
       cmap[i].green = 0x101 * qt_4map[d+1];
