@@ -2,15 +2,15 @@
 /*
  * xa_qt_decs.c
  *
- * Copyright (C) 1996,1997 by Mark Podlipec.
+ * Copyright (C) 1996,1997,1998 by Mark Podlipec.
  * All rights reserved.
  *
- * This software may be freely copied, modified and redistributed without
- * fee for non-commerical purposes provided that this copyright notice is
- * preserved intact on all copies and modified copies.
+ * This software may be freely used, copied and redistributed without
+ * fee for non-commerical purposes provided that this copyright
+ * notice is preserved intact on all copies.
  *
  * There is no warranty or other guarantee of fitness of this software.
- * It is provided solely "as is". The author(s) disclaim(s) all
+ * It is provided solely "as is". The author disclaims all
  * responsibility and liability with respect to this software's usage
  * or its effect upon hardware or computer systems.
  *
@@ -50,6 +50,7 @@ xaULONG QT_Decode_RAW16();
 xaULONG QT_Decode_RAW24();
 xaULONG QT_Decode_RAW32();
 xaULONG QT_Decode_RLE1();
+xaULONG QT_Decode_RLE2();
 xaULONG QT_Decode_RLE4();
 xaULONG QT_Decode_RLE8();
 xaULONG QT_Decode_RLE16();
@@ -138,6 +139,11 @@ XA_CODEC_HDR *codec;
 			codec->x = 16 * ((codec->x + 15)/16);
 			codec->depth = 1;
 			break;
+	  case 34:
+	  case  2:	codec->decoder = QT_Decode_RLE2;
+			codec->x = 16 * ((codec->x + 15)/16);
+			codec->depth = 4;
+			break;
 	  case 36:
 	  case  4:	codec->decoder = QT_Decode_RLE4;
 			codec->x = 8 * ((codec->x + 7)/8);
@@ -207,19 +213,32 @@ XA_CODEC_HDR *codec;
         codec->description      = "JPEG";
 	codec->xapi_rev		= 0x0002;
         ret = CODEC_SUPPORTED;
-	if ((codec->depth == 8) || (codec->depth == 24)
-	    || (codec->depth == 40) )
-	{ JPG_Alloc_MCU_Bufs(codec->anim_hdr,codec->x,0,xaFALSE);
-	  JPG_Setup_Samp_Limit_Table(codec->anim_hdr);
-	  codec->x = 4 * ((codec->x + 3)/4);
-	  codec->y = 2 * ((codec->y + 1)/2);
-	  codec->decoder = JFIF_Decode_JPEG;
-	  XA_Gen_YUV_Tabs(codec->anim_hdr);
-	  if (codec->depth == 40) codec->depth = 8;
-	}
-	else ret = CODEC_UNSUPPORTED;
+	JPG_Alloc_MCU_Bufs(codec->anim_hdr,codec->x,0,xaFALSE);
+	JPG_Setup_Samp_Limit_Table(codec->anim_hdr);
+	codec->x = 4 * ((codec->x + 3)/4);
+	codec->y = 2 * ((codec->y + 1)/2);
+	codec->decoder = JFIF_Decode_JPEG;
+	if (codec->depth == 40) codec->depth = 8;
+        if (codec->depth > 8)   XA_Gen_YUV_Tabs(codec->anim_hdr);
 	break;
 
+/* NOTE: image appears to shifted 25% to the right  qts/mjpg/08*.qt */
+    case QT_MJPG:
+    case QT_mjpg:
+        codec->compression      = QT_MJPG;
+        codec->description      = "Motion JPEG";
+        codec->xapi_rev = 0x0002;
+        ret = CODEC_SUPPORTED;
+        codec->extra    = (void *)(0x00);
+        codec->decoder = JFIF_Decode_JPEG;
+        codec->x = 4 * ((codec->x + 3)/4);
+        codec->y = 2 * ((codec->y + 1)/2);
+	if (codec->depth == 40) codec->depth = 8;
+        if (codec->depth > 8)   XA_Gen_YUV_Tabs(codec->anim_hdr);
+        else    codec->avi_ctab_flag = xaFALSE;
+        JPG_Alloc_MCU_Bufs(codec->anim_hdr,codec->x,0,xaFALSE);
+        break;
+
     case QT_yuv2:
 	codec->xapi_rev = 0x0002;
         codec->compression      = QT_yuv2;
@@ -334,6 +353,12 @@ XA_CODEC_HDR *codec;
         ret = CODEC_UNSUPPORTED;
 	break;
 
+    case QT_SVQ1:
+        codec->compression      = QT_SVQ1;
+        codec->description      = "Sorenson Video";
+        ret = CODEC_UNSUPPORTED;
+	break;
+
     default:
         ret = CODEC_UNKNOWN;
         break;
@@ -761,6 +786,128 @@ XA_DEC_INFO *dec_info;  /* Decoder Info Header */
  else return(ACT_DLTA_NORM);
 }
 
+/********************* * * * *******************************************/
+xaULONG QT_Decode_RLE2(image,delta,dsize,dec_info)
+xaUBYTE *image;         /* Image Buffer. */
+xaUBYTE *delta;         /* delta data. */
+xaULONG dsize;          /* delta size */
+XA_DEC_INFO *dec_info;  /* Decoder Info Header */
+{ xaULONG imagex = dec_info->imagex;    xaULONG imagey = dec_info->imagey;
+  xaULONG map_flag = dec_info->map_flag;        xaULONG *map = dec_info->map;
+  xaLONG x,y,lines; /* xaLONG min_x,max_x,min_y,max_y; */
+  xaULONG d;
+  xaUBYTE *dptr;
+
+  dptr = delta;
+  dptr += 4;    /* skip codec size */
+  d = (*dptr++) << 8;  d |= *dptr++;   /* read code either 0008 or 0000 */
+  if (dsize < 8) /* NOP */
+  { 
+    dec_info->xs = dec_info->ys = dec_info->xe = dec_info->ye = 0;
+DEBUG_LEVEL1 fprintf(stderr,"nop\n"); /* POD */
+    return(ACT_DLTA_NOP);
+  }
+  if (d & 0x0008) /* Header present */
+  {
+    y = (*dptr++) << 8; y |= *dptr++;           /* start line */
+    dptr += 2;                                  /* unknown */
+    lines = (*dptr++) << 8; lines |= *dptr++;   /* number of lines */
+    dptr += 2;                                  /* unknown */
+  }
+  else { y = 0; lines = imagey; }
+
+DEBUG_LEVEL1 fprintf(stderr,"d=%d y=%d lines %d\n",d,y,lines); /* POD */
+
+  x = -1; lines++;
+  while(lines)				/* loop thru lines */
+  {
+    xaULONG xskip,cnt;
+
+    if (x == -1) 
+    {  xskip = *dptr++;  /* skip x pixels */
+       if (xskip == 0) break;  /* end of codec */
+    }
+    else xskip = 0;
+
+
+    cnt = *dptr++;				/* RLE code */
+
+DEBUG_LEVEL1 fprintf(stderr," xy <%d,%d> xskip %x cnt %x\n",x,y,xskip,cnt);
+
+    if (cnt == 0) break; 			/* exit */
+    
+/* this can be removed */
+    if (cnt == 0x00)  /* end of codec */
+    {
+	  lines = 0; y++; x = -1; 
+    }
+    else if (cnt == 0xff) /* skip line */
+    { lines--; y++; x = -1; 
+
+      DEBUG_LEVEL1 fprintf(stderr,"    skip line xy <%d,%d>\n",x,y);
+    }
+    else
+    {
+      if (xskip & 0x80) {lines--; y++; x = xskip & 0x7f;}
+      else x += xskip;
+
+      DEBUG_LEVEL1 fprintf(stderr,"    cnt %x <%d,%d>\n",cnt,x,y);
+
+      if (cnt < 0x80)				/* run of data */
+      { 
+        xaUBYTE *bptr; xaUSHORT *sptr; xaULONG *lptr;
+	if ((x11_bytes_pixel==1) || (map_flag==xaFALSE) )
+		bptr = (xaUBYTE *)(image + (y * imagex) + (x << 4) );
+	else if (x11_bytes_pixel==2)
+		sptr = (xaUSHORT *)(image + 2*(y * imagex) + (x << 5) );
+        else lptr = (xaULONG *)(image + 4*(y * imagex) + (x << 6) );
+        x += cnt;
+        while(cnt--) 
+        { xaULONG i,shift,d0;
+          d0 = (*dptr++ << 24);  d0 |= (*dptr++ << 16); 
+				d0 |= (*dptr++ << 8);  d0 |= *dptr++;
+          shift = 32;
+          for(i=0;i<16;i++)
+          { shift -= 2;
+            d = (d0 >> shift) & 0x03;
+            if (map_flag==xaFALSE)	 *bptr++ = (xaUBYTE)d;
+            else if (x11_bytes_pixel==1) *bptr++ = (xaUBYTE)map[d];
+            else if (x11_bytes_pixel==2) *sptr++ = (xaUSHORT)map[d];
+            else			 *lptr++ = (xaULONG)map[d]; 
+          }
+        }
+      } /* end run */ 
+      else				/* repeat data */
+      { xaULONG d0;
+        xaUBYTE *bptr; xaUSHORT *sptr; xaULONG *lptr;
+	if ((x11_bytes_pixel==1) || (map_flag==xaFALSE) )
+		bptr = (xaUBYTE *)(image + (y * imagex) + (x << 4) );
+	else if (x11_bytes_pixel==2)
+		sptr = (xaUSHORT *)(image + 2*(y * imagex) + (x << 5) );
+        else lptr = (xaULONG *)(image + 4*(y * imagex) + (x << 6) );
+        cnt = 0x100 - cnt;
+        x += cnt;
+        d0 = (*dptr++ << 24);  d0 |= (*dptr++ << 16); 
+				d0 |= (*dptr++ << 8);  d0 |= *dptr++;
+        while(cnt--) 
+        { xaULONG i,shift;
+          shift = 32;
+          for(i=0;i<16;i++)
+          { shift -= 2;
+            d = (d0 >> shift) & 0x03;
+            if (map_flag==xaFALSE) 	 *bptr++ = (xaUBYTE)d;
+            else if (x11_bytes_pixel==1) *bptr++ = (xaUBYTE)map[d];
+            else if (x11_bytes_pixel==2) *sptr++ = (xaUSHORT)map[d];
+            else			 *lptr++ = (xaULONG)map[d]; 
+          }
+        }
+      } /* end repeat */
+    } /* end of code */
+  } /* end of lines */
+ dec_info->xs = dec_info->ys = 0; dec_info->xe = imagex; dec_info->ye = imagey;
+ if (map_flag==xaTRUE) return(ACT_DLTA_MAPD);
+ else return(ACT_DLTA_NORM);
+}
 
 /********************* * * * *******************************************/
 xaULONG QT_Decode_RLE4(image,delta,dsize,dec_info)
@@ -780,7 +927,7 @@ XA_DEC_INFO *dec_info;  /* Decoder Info Header */
   if (dsize < 8) /* NOP */
   { 
     dec_info->xs = dec_info->ys = dec_info->xe = dec_info->ye = 0;
-fprintf(stderr,"nop\n"); /* POD */
+DEBUG_LEVEL1 fprintf(stderr,"nop\n"); /* POD */
     return(ACT_DLTA_NOP);
   }
   if (d & 0x0008) /* Header present */
@@ -792,7 +939,7 @@ fprintf(stderr,"nop\n"); /* POD */
   }
   else { y = 0; lines = imagey; }
 
-fprintf(stderr,"d=%d y=%d lines %d\n",d,y,lines); /* POD */
+DEBUG_LEVEL1 fprintf(stderr,"d=%d y=%d lines %d\n",d,y,lines); /* POD */
 
   x = -1; lines++;
   while(lines)				/* loop thru lines */
@@ -2166,6 +2313,7 @@ xaULONG dsize;          /* delta size */
 XA_DEC2_INFO *dec_info;  /* Decoder Info Header */
 { xaULONG imagex = dec_info->imagex;    xaULONG imagey = dec_info->imagey;
   xaULONG map_flag = dec_info->map_flag;        xaULONG *map = dec_info->map;
+  xaULONG special = dec_info->special & 0x01;
   XA_CHDR *chdr = dec_info->chdr;
   xaUBYTE *dptr = delta;
   xaULONG ycnt, row_inc = imagex << 1;
@@ -2178,7 +2326,7 @@ XA_DEC2_INFO *dec_info;  /* Decoder Info Header */
   if (dec_info->skip_flag > 0) return(ACT_DLTA_DROP);
   if (chdr) { if (chdr->new_chdr) chdr = chdr->new_chdr; }
 
-  row_inc *= dec_info->bytes_pixel;
+  row_inc *= (special)?(3):(dec_info->bytes_pixel);
 
   ycnt = 0;
   while( (imagey > 0) && (dsize > 0) )
