@@ -2,15 +2,15 @@
 /*
  * xa_show.c
  *
- * Copyright (C) 1990,1991,1992,1993,1994,1995,1996,1997 by Mark Podlipec.
+ * Copyright (C) 1990-1997,1998 by Mark Podlipec.
  * All rights reserved.
  *
- * This software may be freely copied, modified and redistributed without
- * fee for non-commerical purposes provided that this copyright notice is
- * preserved intact on all copies and modified copies.
+ * This software may be freely used, copied and redistributed without
+ * fee for non-commerical purposes provided that this copyright
+ * notice is preserved intact on all copies.
  *
  * There is no warranty or other guarantee of fitness of this software.
- * It is provided solely "as is". The author(s) disclaim(s) all
+ * It is provided solely "as is". The author disclaims all
  * responsibility and liability with respect to this software's usage
  * or its effect upon hardware or computer systems.
  *
@@ -18,6 +18,9 @@
 
 /* Revisions:
  * 13Nov94:  wasn't properly skipping video frames in certain instances.
+ * 09Sep97:  Eric Shaw sent mods to tile video to root display. Very Cool.
+ * 14Dec97:  Adam Moss: XA_SHOW_IMAGE routine wasn't freeing PixMap when
+ *           scaling images with clip masks.
  */
 
 #include "xanim.h"
@@ -116,38 +119,98 @@ extern float xa_scaley;
 /* override XPutImage call to support playing at a particular
    position in a window.  Uses #define kludge to avoid changing
    lots of code */
-void xa_putimage(Display *dpy, Drawable d, GC gc, XImage *im,
-                 int sx, int sy, int dx, int dy,
-                 unsigned int w, unsigned int h)
-{
-  if (d == mainW) {
-    dx += xa_window_x;
+void xa_putimage(dpy, d, gc, im, sx, sy, dx, dy, w, h)
+Display *dpy;
+Drawable d;
+GC gc;
+XImage *im;
+int sx, sy, dx, dy;
+unsigned int w;
+unsigned int h;
+{ 
+  /* variables used for tiling */
+  XWindowAttributes	rootwindowattributes;
+  Window	root_return;
+  int		x_return,y_return;
+  unsigned int	width_return,height_return, border_width_return,depth_return;
+  int rowcount;
+  int colcount;
+  int rowindex;
+  int colindex;
+
+  if (d == mainW) 
+  { dx += xa_window_x;
     dy += xa_window_y;
     if (xa_window_center_flag == xaTRUE)
-    {
-      dx -= (int)((double)xa_max_imagex * xa_scalex / 2.0);
+    { dx -= (int)((double)xa_max_imagex * xa_scalex / 2.0);
       dy -= (int)((double)xa_max_imagey * xa_scaley / 2.0);
     }
   }
   XPutImage(dpy, d, gc, im, sx, sy, dx, dy, w, h);
+
+  /* stuff for tiling starts here */
+  if(xa_root == xaTRUE)
+  {
+    XGetGeometry(dpy,d,&root_return,&x_return,&y_return,&width_return,
+                     &height_return,&border_width_return,&depth_return);
+    XGetWindowAttributes(theDisp,root_return,&rootwindowattributes);
+    w += 2*dx;
+    h += 2*dy;
+    colcount=rootwindowattributes.width/w + 1;
+    rowcount=rootwindowattributes.height/h + 1;
+    for(rowindex=0;rowindex<rowcount;rowindex++)
+      for(colindex=0;colindex<colcount;colindex++)
+        XPutImage(theDisp,root_return,theGC,im,sx,sy,
+           w*colindex+dx,h*rowindex+dy,w-2*dx,h-2*dy);
+  }
 }
 #define XPutImage xa_putimage
 
 #ifdef XSHM
-void xa_shmputimage(Display *dpy, Drawable d, GC gc, XImage *im,
-                    int sx, int sy, int dx, int dy,
-                    unsigned int w, unsigned int h, int event)
-{
-  if (d == mainW) {
-    dx += xa_window_x;
+void xa_shmputimage(dpy, d, gc, im, sx, sy, dx, dy, w, h, event)
+Display *dpy;
+Drawable d;
+GC gc;
+XImage *im;
+int sx, sy, dx, dy;
+unsigned int w, h;
+int event;
+{ 
+  /* variables used for tiling */
+  XWindowAttributes rootwindowattributes;
+  Window root_return;
+  int x_return,y_return;
+  unsigned int width_return,height_return,
+     border_width_return,depth_return;
+  int rowcount;
+  int colcount;
+  int rowindex;
+  int colindex;
+
+  if (d == mainW) 
+  { dx += xa_window_x;
     dy += xa_window_y;
     if (xa_window_center_flag == xaTRUE)
-    {
-      dx -= (int)((double)xa_max_imagex * xa_scalex / 2.0);
+    { dx -= (int)((double)xa_max_imagex * xa_scalex / 2.0);
       dy -= (int)((double)xa_max_imagey * xa_scaley / 2.0);
     }
   }
   XShmPutImage(dpy, d, gc, im, sx, sy, dx, dy, w, h, False);
+
+  /* stuff for tiling starts here */
+  if (xa_root == xaTRUE)
+  {
+    XGetGeometry(dpy,d,&root_return,&x_return,&y_return,&width_return,
+                     &height_return,&border_width_return,&depth_return);
+    XGetWindowAttributes(theDisp,root_return,&rootwindowattributes);
+    w += 2*dx; h+= 2*dy;
+    colcount=rootwindowattributes.width/w + 1;
+    rowcount=rootwindowattributes.height/h + 1;
+    for(rowindex=0;rowindex<rowcount;rowindex++)
+      for(colindex=0;colindex<colcount;colindex++)
+        XShmPutImage(theDisp,root_return,theGC,im,sx,sy,
+           w*colindex+dx,h*rowindex+dy,w-2*dx,h-2*dy,False);
+  }
 }
 #define XShmPutImage xa_shmputimage
 #endif
@@ -394,7 +457,10 @@ xaULONG flag;		/* override flag 0 normal 1 use pos/size */
 		XA_Install_CMAP(act->chdr);
       XPutImage(theDisp,mainW,theGC,theImage,0,0,xp,yp,xsize,ysize);
     }
-    if(act_im_hdr->clip) XSetClipMask(theDisp,theGC,None);
+    if(act_im_hdr->clip)
+    { XSetClipMask(theDisp,theGC,None);
+      if (pix_map) XFreePixmap(theDisp,pix_map);
+    }
   }
   else /* Not scaling Image */
   {
