@@ -2,15 +2,15 @@
 /*
  * xa_utils.c
  *
- * Copyright (C) 1991,1992,1993,1994,1995,1996,1997 by Mark Podlipec. 
+ * Copyright (C) 1991-1997,1998 by Mark Podlipec. 
  * All rights reserved.
  *
- * This software may be freely copied, modified and redistributed without
- * fee for non-commerical purposes provided that this copyright notice is
- * preserved intact on all copies and modified copies.
+ * This software may be freely used, copied and redistributed without
+ * fee for non-commerical purposes provided that this copyright
+ * notice is preserved intact on all copies.
  * 
  * There is no warranty or other guarantee of fitness of this software.
- * It is provided solely "as is". The author(s) disclaim(s) all
+ * It is provided solely "as is". The author disclaims all
  * responsibility and liability with respect to this software's usage
  * or its effect upon hardware or computer systems.
  *
@@ -18,6 +18,9 @@
 /* revhist
  * 12dec94  +bCdm caused core on Alpha OSF/1 v3.0. 
  *           found and fixed by David Mosberger-Tang.
+ * 14Dec97  Adam Moss:  Optimized scaling routines and found bug
+ *          when allocating memory for scaling the clip mask of
+ *	    an image(used in GIF animations).
  *
  */ 
 #include "xanim.h"
@@ -831,8 +834,10 @@ DEBUG_LEVEL1 fprintf(stderr,"SCALE BITMAP: pos %dx%d -> %dx%d\n",
   if ( (xsize<<1) > xa_scale_row_size)
   {
     xaULONG *tmp;
-    if (xa_scale_row_buff == 0) tmp = (xaULONG *) malloc(xsize * sizeof(xaULONG) );
-    else tmp = (xaULONG *) realloc(xa_scale_row_buff,(xsize<<1)*sizeof(xaULONG));
+    if (xa_scale_row_buff == 0)
+	tmp = (xaULONG *) malloc((xsize<<1) * sizeof(xaULONG) );
+    else
+	tmp = (xaULONG *) realloc(xa_scale_row_buff,(xsize<<1)*sizeof(xaULONG));
     if (tmp == 0) TheEnd1("UTIL_Scale_Mapped: row malloc err\n");
     xa_scale_row_buff = tmp;    xa_scale_row_size = (xsize<<1);
   }
