@@ -2,15 +2,15 @@
 /*
  * xa_x11.c
  *
- * Copyright (C) 1990,1991,1992,1993,1994,1995,1996,1997 by Mark Podlipec. 
+ * Copyright (C) 1990-1997,1998 by Mark Podlipec. 
  * All rights reserved.
  *
- * This software may be freely copied, modified and redistributed without
- * fee for non-commerical purposes provided that this copyright notice is
- * preserved intact on all copies and modified copies.
+ * This software may be freely used, copied and redistributed without
+ * fee for non-commerical purposes provided that this copyright
+ * notice is preserved intact on all copies.
  * 
  * There is no warranty or other guarantee of fitness of this software.
- * It is provided solely "as is". The author(s) disclaim(s) all
+ * It is provided solely "as is". The author disclaims all
  * responsibility and liability with respect to this software's usage
  * or its effect upon hardware or computer systems.
  *
@@ -437,7 +437,24 @@ xaLONG xa_user_class;
   }
 
   vis_i = xa_user_visual;
-  if (vis_i >= vis_num) TheEnd1("X11: Couldn't get requested Visual.");
+
+  if (vis_i >= vis_num)
+  {  fprintf(stdout,
+        "X11: Couldn't get requested Visual. Using default instead.");
+    vis_i = -1;
+  }
+
+        /* Use Default if None are selected */
+  if ((vis_i < 0) || (vis_i >= vis_num))
+  { int i; VisualID vis_d =
+        XVisualIDFromVisual(DefaultVisual(theDisp, DefaultScreen(theDisp)));
+    i = 0;
+    while(i < vis_num)
+    {  if (vis_d == vis[i].visualid) { vis_i = i; break; }
+       i++;
+    }
+  }
+
   if (vis_i < 0)
   { int screen;
     xaLONG max_csize,max_depth,max_class;
@@ -482,6 +499,9 @@ xaLONG xa_user_class;
   x11_depth = vis[vis_i].depth;
   x11_class = vis[vis_i].class;
   x11_cmap_size   = vis[vis_i].colormap_size;
+   /* for now only use 256 colors even if more avail */
+  if (x11_cmap_size > 256) x11_cmap_size = 256;
+
   /* POD - For testing purposes only */
   if ( (pod_max_colors > 0) && (pod_max_colors < x11_cmap_size) )
 		x11_cmap_size = pod_max_colors;
@@ -514,11 +534,9 @@ xaLONG xa_user_class;
     x11_byte_order = x11_bit_order;
   }
   else 
-  { if (x11_depth > 16)
-        { x11_bytes_pixel = 4; x11_bitmap_pad = 32; }
-    else if (x11_depth > 8)
-        { x11_bytes_pixel = 2; x11_bitmap_pad = 16; }
-    else { x11_bytes_pixel = 1; x11_bitmap_pad = 8; }
+  { if (x11_depth > 16)		x11_bitmap_pad = 32;
+    else if (x11_depth > 8)	x11_bitmap_pad = 16;
+    else			x11_bitmap_pad = 8;
 
     theImage = XCreateImage(theDisp,theVisual,
 			x11_depth,ZPixmap,0,0,7,7,
@@ -536,6 +554,33 @@ xaLONG xa_user_class;
       XDestroyImage(theImage); theImage = NULL;
     } else x11_bits_per_pixel = x11_depth;
 
+
+    switch(x11_bits_per_pixel)
+    {
+      case 32:
+	x11_bytes_pixel = 4;
+	break;
+      case 16:
+	x11_bytes_pixel = 2;
+	break;
+      case  8:
+	x11_bytes_pixel = 1;
+	break;
+      case 24:
+	fprintf(stderr,
+		"X11: Packed 24 bpp is non-standard and not yet supported.\n");
+	fprintf(stderr,
+		"     See the bug section in Rev_History for details.\n");
+	fprintf(stderr,
+		"     Until then an 8, 16 or 32 bpp Visual should work.\n");
+	TheEnd();
+	break;
+      default:
+	fprintf(stderr,"X11: %d bpp non-standard and not yet supported\n");
+	TheEnd();
+	break;
+    }
+
     switch(x11_class)
     { case StaticGray:
 	x11_display_type = XA_STATICGRAY;
@@ -749,6 +794,7 @@ xaLONG startx,starty;
    Note that shell window is never realized. */
   if (xa_window_id != 0)
   {
+	/* Check to see if it exists ??? */
     mainW = (Window)xa_window_id;
   }
   else
@@ -790,9 +836,8 @@ xaLONG startx,starty;
     gc_init.graphics_exposures = False;         gc_mask |= GCGraphicsExposures;
     theGC  = XCreateGC(theDisp,mainW,gc_mask,&gc_init);
 
-XSetFunction(theDisp, theGC, GXcopy);
-XSetPlaneMask(theDisp, theGC, ~0L);
-
+    XSetFunction(theDisp, theGC, GXcopy);
+    XSetPlaneMask(theDisp, theGC, ~0L);
   }
 
 
@@ -1432,6 +1477,9 @@ void xanim_events()
 	    vaudiof->newvol = xaTRUE;
 	    XA_AUDIO_SET_VOLUME(vaudiof->volume);
 	    break;
+	  case 'z':
+	    xanim_realize_remote(theWG, event, NULL, 0);
+	    break;
 	  case 'e':
 	    x11_expose_flag = xaTRUE;
 /* Need a function to redraw window here! */
@@ -1509,8 +1557,9 @@ void X11_Show_Visuals()
   {
     for(i=0;i<vis_num;i++)
     {
-      fprintf(stderr,"  visual %d) depth= %d  class= %d  cmap size=%d  ",
-                       i, vis[i].depth, vis[i].class, vis[i].colormap_size );
+      fprintf(stderr,"  visual %d) depth= %d  class= %d  cmap size=%d  bitsrgb %d",
+                       i, vis[i].depth, vis[i].class, vis[i].colormap_size,
+			vis[i].bits_per_rgb  );
       X11_OutPut_Visual_Class(vis[i].class);
       fprintf(stderr,"\n");
     }
@@ -2402,22 +2451,22 @@ void XA_Remote_Free()
  ****************************************************************************/
 void xa_remote_expose();
 
-#include "BM_step_prev.xbm"
-#include "BM_play.xbm"
-#include "BM_step_next.xbm"
-#include "BM_next.xbm"
-#include "BM_quit.xbm"
-#include "BM_prev.xbm"
-#include "BM_slower.xbm"
-#include "BM_speed1.xbm"
-#include "BM_faster.xbm"
-#include "BM_vol_dn.xbm"
-#include "BM_vol_off.xbm"
-#include "BM_vol_up.xbm"
-#include "BM_back.xbm"
-#include "BM_stop.xbm"
-#include "BM_vol_on.xbm"
-#include "BM_fuzz.xbm"
+#include "buttons/BM_step_prev.xbm"
+#include "buttons/BM_play.xbm"
+#include "buttons/BM_step_next.xbm"
+#include "buttons/BM_next.xbm"
+#include "buttons/BM_quit.xbm"
+#include "buttons/BM_prev.xbm"
+#include "buttons/BM_slower.xbm"
+#include "buttons/BM_speed1.xbm"
+#include "buttons/BM_faster.xbm"
+#include "buttons/BM_vol_dn.xbm"
+#include "buttons/BM_vol_off.xbm"
+#include "buttons/BM_vol_up.xbm"
+#include "buttons/BM_back.xbm"
+#include "buttons/BM_stop.xbm"
+#include "buttons/BM_vol_on.xbm"
+#include "buttons/BM_fuzz.xbm"
 
 #define BM_NUMBER 16
 #define BM_STPP		0
