@@ -2,15 +2,15 @@
 /*
  * xanim.h
  *
- * Copyright (C) 1990,1991,1992,1993,1994,1995,1996,1997 by Mark Podlipec. 
+ * Copyright (C) 1990-1997,1998 by Mark Podlipec. 
  * All rights reserved.
  *
- * This software may be freely copied, modified and redistributed without
- * fee for non-commerical purposes provided that this copyright notice is
- * preserved intact on all copies and modified copies.
+ * This software may be freely used, copied and redistributed without
+ * fee for non-commerical purposes provided that this copyright
+ * notice is preserved intact on all copies.
  * 
  * There is no warranty or other guarantee of fitness of this software.
- * It is provided solely "as is". The author(s) disclaim(s) all
+ * It is provided solely "as is". The author disclaims all
  * responsibility and liability with respect to this software's usage
  * or its effect upon hardware or computer systems.
  *
@@ -134,6 +134,7 @@ extern xaLONG x11_black;
 extern xaLONG x11_white;
 extern xaLONG x11_verbose_flag;
 extern xaULONG x11_kludge_1;
+extern xaLONG xa_root;
 
 #define X11_MSB  1
 #define X11_LSB  0
@@ -342,6 +343,8 @@ typedef struct
 #define XA_AUDIO_ARMLAW		0x000060
 #define XA_AUDIO_IMA4 		0x000070
 #define XA_AUDIO_DVI  		0x000080
+#define XA_AUDIO_GSM  		0x000090
+#define XA_AUDIO_MSGSM 		0x0000A0
 #define XA_AUDIO_TYPE_MASK  	0xfffff0
 
 /*NOTES: 
@@ -378,6 +381,9 @@ typedef struct
 #define XA_AUDIO_DVI_M		0x000080
 #define XA_AUDIO_DVI_S		0x000081
 
+#define XA_AUDIO_GSM_M		0x000090
+#define XA_AUDIO_MSGSM_M	0x0000A0
+
 #define XA_AUDIO_FILE_FLAG   0x0001
 
 typedef struct XA_SND_STRUCT
